/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Customer;
import com.ibm.websphere.icp.industry.db.entities.BaseObject;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class CheckOFACImpl {
    /**
     * Default constructor.
     */
    public CheckOFACImpl() {
        super();
    }

    private static final String RESTRICTED_CLASS_MGMT = "http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/RestrictedClassificationManagement";

    private static Log _log = LogFactory.getLog(CheckOFACImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "retrieveDesignatedPersonsList" defined for WSDL port type named
     * "RetrieveDesignatedPersonsListPT".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject retrieveDesignatedPersonsList(DataObject aRetrieveDesignatedPersonsListRequest) {

        _log.info("Inside retrieveDesignatedPersonsList method of CheckOFACImpl");

        DataObject requestHeader = aRetrieveDesignatedPersonsListRequest.getDataObject("requestHeader");

        DataObject responseDO = IndustrySupportUtil.createDO(RESTRICTED_CLASS_MGMT,
                "RetrieveDesignatedPersonsListResponse");
        DataObject responseHeader = responseDO.createDataObject("responseHeader");

        String identifier = null;
        if (requestHeader != null) {
            identifier = requestHeader.getString("id");
        }

        String ofacStatus = "ok";
        if (StringUtils.isNotBlank(identifier)) {

            responseHeader.setString("id", identifier);

            NAOUtility utility = NAOUtility.getInstance();
            IWicpDBManager dbManager = NAOUtility.getNaoDBManager();

            // get customer details
            Map<String, Object> params = new HashMap<String, Object>();
            params.put("customerId", Integer.parseInt(identifier));
            List<BaseObject> customers = dbManager.executeNamedQuery("findCustomerById", params);

            String lastName = null;

            if (customers != null && !customers.isEmpty()) {
                // expecting only one entity
                Customer customer = (Customer) customers.get(0);

                lastName = customer.getLastname();
            }

            if (StringUtils.isNotBlank(lastName) && lastName.toLowerCase().endsWith("xyz")) {
                ofacStatus = "fail";
            }
        } else {
            ofacStatus = "fail";
        }

        responseHeader.setString("cmdStatus", ofacStatus);
        return responseDO;
    }
}