/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import commonj.sdo.DataObject;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Arrangement;
import com.ibm.websphere.icp.banking.nao.entity.Customer;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.sca.ServiceManager;

public class CreateArrangementImpl {
    /**
     * Default constructor.
     */
    public CreateArrangementImpl() {
        super();
    }

    private static final String FINANCE_SERVICE_NS = "http://www.ibm.com/industries/financialservices";
    private static final String CREATE_ARRANGEMENT_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/ArrangementSetup";

    private static final Log _log = LogFactory.getLog(CreateArrangementImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "recordArrangementDetails" defined for WSDL port type named
     * "RecordArrangementDetailsPT".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject recordArrangementDetails(DataObject aRecordArrangementDetailsRequest) {
        _log.info("Inside recordArrangementDetails operation of CreateArrangementImpl");

        DataObject arrangementBO = aRecordArrangementDetailsRequest.getDataObject("arrangement");
        
        //IndustrySupportUtil.printBO(arrangementBO);
        
        if (arrangementBO != null) {
            NAOUtility utility = NAOUtility.getInstance();

            Arrangement arrangement = utility.createArrangementEntity(arrangementBO);

            //initialize lifecycle status, putting the request by default on review
            arrangement.setLifecyclestatus(DataSetup.LifecycleStatus.SUSPENDED);
            
            IWicpDBManager dbManager = NAOUtility.getNaoDBManager();
            
            boolean update = arrangement.getCustomerId().getCustomerId() > 0 ? true : false;
           
            dbManager.save(arrangement);
            
            if(update) {
            dbManager.update(arrangement.getCustomerId());
           
            }
            

            // update arrangement BO with application reference number and
            // customer Id
            updateArrangementBO(arrangementBO, arrangement);
        }

        DataObject responseDO = IndustrySupportUtil.createDO(CREATE_ARRANGEMENT_NS, "RecordArrangementDetailsResponse");
        DataObject responseHeader = IndustrySupportUtil.createDO(FINANCE_SERVICE_NS, "ResponseHeader");
        responseDO.setDataObject("responseHeader", responseHeader);
        responseHeader.setString("id",arrangementBO.getDataObject("customer").getString("customerId")+"/"+arrangementBO.getString("applicationNumber"));
        return responseDO;
    }

    private void updateArrangementBO(DataObject arrangementBO, Arrangement arrangement) {
        // set application reference number
        if (arrangement.getApplicationnumber() != null) {
            arrangementBO.setString("applicationNumber", arrangement.getApplicationnumber());
        }

        // set customer id
        Customer customer = arrangement.getCustomerId();
        //System.out.println("id:"+customer.getCustomerId());
        arrangementBO.getDataObject("customer").setString("customerId", String.valueOf(customer.getCustomerId()));
    }

}