/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class DocumentCheckListImpl {
    /**
     * Default constructor.
     */
    public DocumentCheckListImpl() {
        super();
    }

    private static final String DOCUMENT_CHECKLIST_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema/DocumentCheckList";

    private static final Log _log = LogFactory.getLog(DocumentCheckListImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "identifyDocumentCheckList" defined for WSDL port type named
     * "DocumentCheckListPT".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject identifyDocumentCheckList(DataObject documentCheckListRequest) {
        _log.info("Inside identityDocumentCheckList operation of DocumentCheckListImpl");

        boolean newCustomer = false;
        String residenceStatusOfUS = null;
        String accountType = null;

        DataObject arrangementDO = documentCheckListRequest.getDataObject("arrangement");
        DataObject customerDO = arrangementDO.getDataObject("customer");

        newCustomer = customerDO.getBoolean("newCustomer");
        _log.info("newCusotmer:" + newCustomer);

        residenceStatusOfUS = customerDO.getString("residenceStatusOfUS");
        _log.info("residenceStatusOfUS:" + residenceStatusOfUS);

        accountType = arrangementDO.getString("accountType");
        _log.info("accountType:" + accountType);

        DataObject responseDO = IndustrySupportUtil.createDO(DOCUMENT_CHECKLIST_NS, "DocumentCheckListResponse");

        // get lists of documents, run rules
        List<String> documents = runRules(newCustomer, residenceStatusOfUS, accountType);
        arrangementDO.setList("documents", documents);

        responseDO.setDataObject("arrangement", arrangementDO);
        return responseDO;
    }

    private List<String> runRules(boolean newCustomer, String residenceStatusOfUS, String accountType) {

        List<String> documents = new ArrayList<String>();
        if (newCustomer) {
            if (residenceStatusOfUS.equals(DataSetup.ResidenceStatusOfUS.CITIZEN_PR)) {
                if (accountType.equals(DataSetup.AccountType.SAVING)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.CITIZENSHIP_PR_PROOF);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                } else if (accountType.equals(DataSetup.AccountType.CD)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.TWO_YRS_IT_RETURNS);
                    documents.add(DataSetup.DocumentTypes.TAX_RETURNS);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.CITIZENSHIP_PR_PROOF);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                } else if (accountType.equals(DataSetup.AccountType.CHECKING)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.CITIZENSHIP_PR_PROOF);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                }
            } else if (residenceStatusOfUS.equals(DataSetup.ResidenceStatusOfUS.TEMPORARY)) {
                if (accountType.equals(DataSetup.AccountType.SAVING)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.VISA_INFORMATION);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                } else if (accountType.equals(DataSetup.AccountType.CD)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.TWO_YRS_IT_RETURNS);
                    documents.add(DataSetup.DocumentTypes.TAX_RETURNS);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.VISA_INFORMATION);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                }else if (accountType.equals(DataSetup.AccountType.CHECKING)) {
                    documents.add(DataSetup.DocumentTypes.SSN_CARD);
                    documents.add(DataSetup.DocumentTypes.DRIVING_LICENSE);
                    documents.add(DataSetup.DocumentTypes.VISA_INFORMATION);
                    documents.add(DataSetup.DocumentTypes.W2_FORM);
                }
            }
        } else {
            if (residenceStatusOfUS.equals(DataSetup.ResidenceStatusOfUS.TEMPORARY)) {
                documents.add(DataSetup.DocumentTypes.VISA_INFORMATION);
            }
        }
        return documents;
    }
}