/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class EvaluateCustomerRiskImpl {
    /**
     * Default constructor.
     */
    public EvaluateCustomerRiskImpl() {
        super();
    }

    private static final String EVAL_CUSTOMER_RISK_NS = "http://www.ibm.com/industries/financialservices";

    private static final String ARRANGEMENT_EXTN_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/Arrangement";

    private static Log _log = LogFactory.getLog(EvaluateCustomerRiskImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "evaluateCustomerRisk" defined for WSDL port type named
     * "OperationalRiskManagement".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject evaluateCustomerRisk(DataObject aEvaluateCustomerRiskRequest) {

        _log.info("Inside evaluateCustomerRisk operation of EvaluateCustomerRiskImpl");

        DataObject customerBO = aEvaluateCustomerRiskRequest.getDataObject("customerDetails");

        String residenceStatusOfUS = null;
        if (customerBO != null) {
            residenceStatusOfUS = customerBO.getString("residenceStatusOfUS");
        }

        boolean customerRisk = false;

        if (DataSetup.ResidenceStatusOfUS.TEMPORARY.equals(residenceStatusOfUS)) {
            customerRisk = true;
        }

        DataObject responseDO = IndustrySupportUtil.createDO(EVAL_CUSTOMER_RISK_NS, "EvaluateCustomerRiskResponse");
        DataObject arrangementRiskDO = IndustrySupportUtil.createDO(ARRANGEMENT_EXTN_NS, "ArrangementRiskExtn");

        if (customerRisk) {
            arrangementRiskDO.setString("riskRating", "High");
        } else {
            arrangementRiskDO.setString("riskRating", "Low");
        }
        responseDO.setDataObject("customerRiskRating", arrangementRiskDO);
        return responseDO;
    }
}