/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Arrangement;
import com.ibm.websphere.icp.banking.nao.entity.Arrangementdocuments;
import com.ibm.websphere.icp.banking.nao.entity.ArrangementdocumentsPK;
import com.ibm.websphere.icp.banking.nao.entity.Contactdetails;
import com.ibm.websphere.icp.banking.nao.entity.Customer;
import com.ibm.websphere.icp.banking.nao.entity.Employmentdetails;
import com.ibm.websphere.icp.banking.nao.entity.Postaladdress;
import com.ibm.websphere.icp.banking.nao.entity.Residencedetails;
import com.ibm.websphere.icp.banking.nao.entity.Term;
import com.ibm.websphere.icp.industry.db.entities.BaseObject;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBFactoryConstants;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.icp.industry.db.manager.WicpDBFactory;

import commonj.sdo.DataObject;

public class NAOUtility {

    private static NAOUtility _instance = null;

    private NAOUtility() {

    }

    private static final String FINANCIAL_SVC_COMMON_NS = "http://www.ibm.com/industries/financialservices";

    private static final String INVOLVED_PARTY_EXTN_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty";

    @SuppressWarnings("unused")
    private static final String ARRANGEMENT_EXTN_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/Arrangement";

    public static NAOUtility getInstance() {
        if (_instance == null) {
            _instance = new NAOUtility();
        }
        return _instance;
    }

    public Arrangement createArrangementEntity(DataObject arrangementBO) {

        Arrangement arrangement = new Arrangement();

        if (arrangementBO != null) {

            // account type
            String accountType = arrangementBO.getString("accountType");
            if (StringUtils.isNotEmpty(accountType)) {
                arrangement.setAccounttype(accountType);
            }

            // product
            String product = arrangementBO.getString("product");
            if (StringUtils.isNotEmpty(product)) {
                arrangement.setProduct(product);
            }

            // operating instruction
            String operationInstruction = arrangementBO.getString("operatingInstruction");
            if (StringUtils.isNotEmpty(operationInstruction)) {
                arrangement.setOperatinginstruction(operationInstruction);
            }

            // card
            String card = arrangementBO.getString("card");
            if (StringUtils.isNotEmpty(card)) {
                arrangement.setCard(card);
            }

            // internet banking
            if (arrangementBO.getBoolean("internetBanking")) {
                arrangement.setInternetbanking("Y");
            } else {
                arrangement.setInternetbanking("N");
            }

            // ivr
            if (arrangementBO.getBoolean("ivr")) {
                arrangement.setIvr("Y");
            } else {
                arrangement.setIvr("N");
            }

            // check book
            if (arrangementBO.getBoolean("checkBook")) {
                arrangement.setCheckbook("Y");
            } else {
                arrangement.setCheckbook("N");
            }

            // check quantity
            arrangement.setCheckquantity(arrangementBO.getInt("checkQuantity"));

            // customer
            DataObject customerBO = arrangementBO.getDataObject("customer");
            if (customerBO != null) {
                Customer customer = createCustomerEntity(customerBO);
                arrangement.setCustomerId(customer);
            }

            // co-applicants
            List<DataObject> coApplicantList = arrangementBO.getList("coApplicants");
            if (coApplicantList != null && !coApplicantList.isEmpty()) {

                Set<Customer> coApplicantSet = new HashSet<Customer>();
                for (DataObject coApplicantBO : coApplicantList) {
                    Customer coApplicant = createCustomerEntity(coApplicantBO);
                    if (coApplicant != null) {
                        coApplicantSet.add(coApplicant);
                    }
                }

                arrangement.setCustomerCollection(coApplicantSet);
            }

            // lifecycle status
            String lifecycleStatus = arrangementBO.getString("lifecycleStatus");
            if (StringUtils.isNotEmpty(lifecycleStatus)) {
                arrangement.setLifecyclestatus(lifecycleStatus);
            }

            // lifecycle phase initiation
            String lifecyclePhaseInitiation = arrangementBO.getString("lifecyclePhaseInitiation");
            if (StringUtils.isNotEmpty(lifecyclePhaseInitiation)) {
                arrangement.setLifecyclephaseinitiation(lifecyclePhaseInitiation);
            }

            // documents
            List<String> documentList = arrangementBO.getList("documents");
            if (documentList != null && !documentList.isEmpty()) {

                Set<Arrangementdocuments> documentsSet = new HashSet<Arrangementdocuments>();
                for (String documentStr : documentList) {
                    Arrangementdocuments document = new Arrangementdocuments();
                    document.setDocument(documentStr);
                    document.setPk(new ArrangementdocumentsPK());

                    documentsSet.add(document);
                }

                arrangement.setArrangementdocumentsCollection(documentsSet);
            }

            // term
            if (accountType.equals(DataSetup.AccountType.CD)) {
                DataObject termBO = arrangementBO.getDataObject("term");
                if (termBO != null) {
                    Term term = new Term();

                    // years
                    term.setYears(termBO.getInt("years"));

                    // months
                    term.setMonths(termBO.getInt("months"));

                    // days
                    term.setDays(termBO.getInt("days"));

                    Set<Term> terms = new HashSet<Term>();
                    terms.add(term);

                    arrangement.setTermCollection(terms);
                }
            }

            // application number
            udpateApplicationNumber(arrangement);

           
            // account statement
            DataObject statementBO = arrangementBO.getDataObject("statement");
            if (statementBO != null) {

                // statement type
                String type = statementBO.getString("type");
                if (StringUtils.isNotEmpty(type)) {
                    arrangement.setStatementtype(type);
                }

                // statement frequency
                String frequency = statementBO.getString("frequency");
                if (StringUtils.isNotEmpty(frequency)) {
                    arrangement.setStatementfrequency(frequency);
                }
            }
            // frequency

        }

        return arrangement;
    }

    private void udpateApplicationNumber(Arrangement arrangement) {
        String applicationNumber = null;
        if (arrangement != null) {
            java.util.Date today = Calendar.getInstance().getTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddhhmmss");
            String todayTxt = sdf.format(today);

            String str = null;
            String accountType = arrangement.getAccounttype();
            //System.out.println("accountType:"+accountType);
            if (accountType.equals(DataSetup.AccountType.CHECKING)) {
                str = "CA";
            } else if (accountType.equals(DataSetup.AccountType.SAVING)) {
                str = "SA";
            } else if (accountType.equals(DataSetup.AccountType.CD)) {
                str = "CD";
            }
            applicationNumber = str + todayTxt;

            arrangement.setApplicationnumber(applicationNumber);
        }
    }

    /**
     * Method transforms CustomerExtn DataObject to Customer entity.
     * 
     * @param customerBO
     *            Customer DataObject
     * @return Customer
     */
    public Customer createCustomerEntity(DataObject customerBO) {
        Customer customer = null;
        
        String customerId = customerBO.getString("customerId");

        if (StringUtils.isEmpty(customerId)) {
        	customer = new Customer();
        }
        else {
        	IWicpDBManager dbManager = getNaoDBManager();
        	
            Map<String, Object> paramMaps = new HashMap<String, Object>();
            paramMaps.put("customerId", Integer.parseInt(customerId));
            
        	List<BaseObject> customers = dbManager.executeNamedQuery("findCustomerById", paramMaps);
        	customer = (Customer) customers.get(0);
        }
        
        // new customer
        if (customerBO.getBoolean("newCustomer")) {
            customer.setNewcustomer("Y");
        } else {
            customer.setNewcustomer("N");
        }

        // gender
        String gender = customerBO.getString("gender");
        if (StringUtils.isNotEmpty(gender)) {
            customer.setGender(gender);
        }

        // marital status
        String maritalStatus = customerBO.getString("maritalStatus");
        if (StringUtils.isNotEmpty(maritalStatus)) {
            customer.setMaritalstatus(maritalStatus);
        }

        // ssn
        String ssn = customerBO.getString("ssn");
        if (StringUtils.isNotEmpty(ssn)) {
            customer.setSsn(ssn);
        }

        // citizenship Country
        String citizenshipCountry = customerBO.getString("citizenshipCountry");
        if (StringUtils.isNotEmpty(citizenshipCountry)) {
            customer.setCitizenshipcountry(citizenshipCountry);
        }

        // relationship manager
        String relationshipManager = customerBO.getString("relationshipManager");
        if (StringUtils.isNotEmpty(relationshipManager)) {
            customer.setRelationshipmanager(relationshipManager);
        }

        // relationship code
        String relationshipCode = customerBO.getString("relationshipCode");
        if (StringUtils.isNotEmpty(relationshipCode)) {
            customer.setRelationshipcode(relationshipCode);
        }

        // customer status
        String customerStatus = customerBO.getString("customerStatus");
        if (StringUtils.isNotEmpty(customerStatus)) {
            customer.setCustomerstatus(customerStatus);
        }

        // age
        customer.setAge(customerBO.getInt("age"));
        // customer name
        DataObject customerNameBO = customerBO.getDataObject("customerName");
        if (customerNameBO != null) {

            // title
            String title = customerNameBO.getString("prefixTitle");
            if (StringUtils.isNotEmpty(title)) {
                customer.setPrefixtitle(title);
            }

            // first name
            String firstName = customerNameBO.getString("firstName");
            if (StringUtils.isNotEmpty(firstName)) {
                customer.setFirstname(firstName);
            }

            // last name
            String lastName = customerNameBO.getString("lastName");
            if (StringUtils.isNotEmpty(lastName)) {
                customer.setLastname(lastName);
            }

            // suffix
            String suffix = customerNameBO.getString("suffixTitle");
            if (StringUtils.isNotEmpty(suffix)) {
                customer.setSuffixtitle(suffix);
            }
        }

        // residence status of US
        String residenceStatusOfUS = customerBO.getString("residenceStatusOfUS");
        if (StringUtils.isNotEmpty(residenceStatusOfUS)) {
            customer.setResidencestatusofus(residenceStatusOfUS);
        }

        // DOB
        java.util.Date dateOfBirth = customerBO.getDate("dateOfBirth");
        if (dateOfBirth != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String dateString = sdf.format(dateOfBirth);
            customer.setDateofbirth(Date.valueOf(dateString));
        }

        // contact details
        DataObject contactDetailsBO = customerBO.getDataObject("contactDetails");
        if (contactDetailsBO != null) {
            Contactdetails contactdetails = null;
            
            if (StringUtils.isNotEmpty(customerId)) {
            	Set<Contactdetails> contactDetailsSet = customer.getContactdetailsCollection();
            	contactdetails = contactDetailsSet.iterator().next();
            }
            
            contactdetails = createUpdateContactDetailsEntity(contactDetailsBO, contactdetails);

            Set<Contactdetails> conDtlsSet = new HashSet<Contactdetails>();
            conDtlsSet.add(contactdetails);

            customer.setContactdetailsCollection(conDtlsSet);
        }

        // residence details
        DataObject residenceDetailsBO = customerBO.getDataObject("residenceDetails");
        if (residenceDetailsBO != null) {
            Residencedetails residencedetails = null;
            
            if (StringUtils.isNotEmpty(customerId)) {
            	Set<Residencedetails> residenceDetailsSet = customer.getResidencedetailsCollection();
            	residencedetails = residenceDetailsSet.iterator().next();
            }
            
            residencedetails = createUpdateResidenceDetailsEntiry(residenceDetailsBO, residencedetails);

            Set<Residencedetails> resDtlsSet = new HashSet<Residencedetails>();
            resDtlsSet.add(residencedetails);
            customer.setResidencedetailsCollection(resDtlsSet);
        }

        // employment details
        DataObject employmentDetailsBO = customerBO.getDataObject("employmentDetails");
        if (employmentDetailsBO != null) {
            Employmentdetails employmentdetails = null;
            
            if (StringUtils.isNotEmpty(customerId)) {
            	Set<Employmentdetails> employmentdetailsSet = customer.getEmploymentdetailsCollection();
            	employmentdetails = employmentdetailsSet.iterator().next();
            }
            
            employmentdetails = createUpdateEmploymentDetailsEntity(employmentDetailsBO, employmentdetails);

            Set<Employmentdetails> emplDtlsSet = new HashSet<Employmentdetails>();
            emplDtlsSet.add(employmentdetails);
            customer.setEmploymentdetailsCollection(emplDtlsSet);
        }

        // postal address
        DataObject postalAddressBO = customerBO.getDataObject("legalAddress");
        if (postalAddressBO != null) {
            Postaladdress postaladdress = null;
            if (StringUtils.isNotEmpty(customerId)) {
            	postaladdress = customer.getPostaladdressId();
            }
            
            postaladdress = createUpdatePostalAddressEntity(postalAddressBO, postaladdress);
            
            customer.setPostaladdressId(postaladdress);
        }

        return customer;
    }

    private Employmentdetails createUpdateEmploymentDetailsEntity(DataObject employmentDetailsBO, Employmentdetails existingEmplDtls) {
        Employmentdetails employmentdetails = null;
        
        if (existingEmplDtls == null) {
			employmentdetails = new Employmentdetails();
		} else {
			employmentdetails = existingEmplDtls;
		}

        if (employmentDetailsBO != null) {

            // employer name
            String employerName = employmentDetailsBO.getString("employerName");
            if (StringUtils.isNotEmpty(employerName)) {
                employmentdetails.setEmployername(employerName);
            }

            // type
            String type = employmentDetailsBO.getString("type");
            if (StringUtils.isNotEmpty(type)) {
                employmentdetails.setType(type);
            }

            // position
            String position = employmentDetailsBO.getString("position");
            if (StringUtils.isNotEmpty(position)) {
                employmentdetails.setPosition(position);
            }

            // annual income
            Long annualIncome = employmentDetailsBO.getLong("annualIncome");
            if (annualIncome != null) {
                employmentdetails.setAnnualincome(annualIncome);
            }

            // home phone number
            String homePhoneNumber = employmentDetailsBO.getString("homePhoneNumber");
            if (StringUtils.isNotEmpty(homePhoneNumber)) {
                employmentdetails.setHomephonenumber(homePhoneNumber);
            }

            // mobile
            String mobile = employmentDetailsBO.getString("mobile");
            if (StringUtils.isNotEmpty(mobile)) {
                employmentdetails.setMobile(mobile);
            }

            // postal address
            DataObject postalAddressBO = employmentDetailsBO.getDataObject("legalAddress");
            if (postalAddressBO != null) {
                Postaladdress postaladdress = employmentdetails.getPostaladdressId();
                postaladdress = createUpdatePostalAddressEntity(postalAddressBO, postaladdress);
                employmentdetails.setPostaladdressId(postaladdress);
            }
        }
        return employmentdetails;
    }

    private Postaladdress createUpdatePostalAddressEntity(DataObject postalAddressBO, Postaladdress existingPostalAddress) {
        Postaladdress postaladdress = null;
       
        if (existingPostalAddress == null) {
			postaladdress = new Postaladdress();
		} else {
			postaladdress = existingPostalAddress;
		}

        if (postalAddressBO != null) {

            List<DataObject> addressComponents = postalAddressBO.getList("addressComponents");
            if (addressComponents != null && !addressComponents.isEmpty()) {

                // addressline1
                String addressline1 = getAddressComponentValueForType(addressComponents,
                        DataSetup.AddressComponentTypes.ADDRESS_LINES, 0);
                if (StringUtils.isNotEmpty(addressline1)) {
                    postaladdress.setAddressline1(addressline1);
                }

                // addressline2
                String addressline2 = getAddressComponentValueForType(addressComponents,
                        DataSetup.AddressComponentTypes.ADDRESS_LINES, 1);
                if (StringUtils.isNotEmpty(addressline2)) {
                    postaladdress.setAddressline2(addressline2);
                }

                // city
                String city = getAddressComponentValueForType(addressComponents, DataSetup.AddressComponentTypes.CITY,
                        0);
                if (StringUtils.isNotEmpty(city)) {
                    postaladdress.setCity(city);
                }

                // postal code
                String postalcode = getAddressComponentValueForType(addressComponents,
                        DataSetup.AddressComponentTypes.POSTAL_CODE, 0);
                if (StringUtils.isNotEmpty(postalcode)) {
                    postaladdress.setPostalcode(postalcode);
                }

                // country
                String country = getAddressComponentValueForType(addressComponents,
                        DataSetup.AddressComponentTypes.COUNTRY, 0);
                if (StringUtils.isNotEmpty(country)) {
                    postaladdress.setCountry(country);
                }

                // state
                String state = getAddressComponentValueForType(addressComponents,
                        DataSetup.AddressComponentTypes.REGION, 0);
                if (StringUtils.isNotEmpty(state)) {
                    postaladdress.setState(state);
                }
            }

        }
        return postaladdress;
    }

    private String getAddressComponentValueForType(List<DataObject> addressComponents, String type, int occurance) {
        String value = null;

        int counter = 0;
        String typeLocal = null;
        if (addressComponents != null && !addressComponents.isEmpty()) {
            for (DataObject component : addressComponents) {
                typeLocal = component.getString("type");
                if (StringUtils.isNotEmpty(typeLocal)) {
                    if (type.equals(typeLocal)) {
                        if (occurance > counter) {
                            counter++;
                            continue;
                        } else {
                            value = component.getString("value");
                            break;
                        }
                    }
                }
            }
        }
        return value;
    }

    private Residencedetails createUpdateResidenceDetailsEntiry(DataObject residenceDetailsBO, Residencedetails existingResidenceDetails) {
        Residencedetails residencedetails = null;
        
        if (existingResidenceDetails == null) {
			residencedetails = new Residencedetails();
		} else {
			residencedetails = existingResidenceDetails;
		}

        if (residenceDetailsBO != null) {

            // country
            String country = residenceDetailsBO.getString("country");
            if (StringUtils.isNotEmpty(country)) {
                residencedetails.setCountry(country);
            }

            // years
            String years = residenceDetailsBO.getString("years");
            if (StringUtils.isNotEmpty(years)) {
                residencedetails.setYears(years);
            }

            // status
            String status = residenceDetailsBO.getString("status");
            if (StringUtils.isNotEmpty(status)) {
                residencedetails.setStatus(status);
            }
        }
        return residencedetails;
    }

    private Contactdetails createUpdateContactDetailsEntity(DataObject contactDetailsBO, Contactdetails existingContactDetails) {
        Contactdetails contactdetails = null;

        if (existingContactDetails == null) {
			contactdetails = new Contactdetails();
		} else {
			contactdetails = existingContactDetails;
		}

        if (contactDetailsBO != null) {

            // home phone number
            String homePhoneNumber = contactDetailsBO.getString("homePhoneNumber");
            if (StringUtils.isNotEmpty(homePhoneNumber)) {
                contactdetails.setHomephonenumber(homePhoneNumber);
            }

            // mobile
            String mobile = contactDetailsBO.getString("mobile");
            if (StringUtils.isNotEmpty(mobile)) {
                contactdetails.setMobile(mobile);
            }

            // email
            String email = contactDetailsBO.getString("email");
            if (StringUtils.isNotEmpty(email)) {
                contactdetails.setEmail(email);
            }

            // preferred language
            String preferredLanguage = contactDetailsBO.getString("preferredLanguage");
            if (StringUtils.isNotEmpty(preferredLanguage)) {
                contactdetails.setPreferredlanguage(preferredLanguage);
            }

            // preferred medium
            String preferredMedium = contactDetailsBO.getString("preferredMedium");
            if (StringUtils.isNotEmpty(preferredMedium)) {
                contactdetails.setPreferredmedium(preferredMedium);
            }

            // preferred time period
            String preferredTimePeriod = contactDetailsBO.getString("preferredTimePeriod");
            if (StringUtils.isNotEmpty(preferredTimePeriod)) {
                contactdetails.setPreferredtimeperiod(preferredTimePeriod);
            }
        }
        return contactdetails;
    }

    public DataObject createCustomerBO(Customer customer) {
        DataObject customerBO = IndustrySupportUtil.createDO(INVOLVED_PARTY_EXTN_NS, "CustomerExtn");

        if (customer != null) {

            // customer id
            customerBO.setString("customerId", String.valueOf(customer.getCustomerId()));

            // customer name
            DataObject customerNameBO = IndustrySupportUtil.createDO(FINANCIAL_SVC_COMMON_NS, "IndividualName");

            // first name
            if (customer.getFirstname() != null) {
                customerNameBO.setString("firstName", customer.getFirstname());
            }

            // last name
            if (customer.getLastname() != null) {
                customerNameBO.setString("lastName", customer.getLastname());
            }

            // prefix
            if (customer.getPrefixtitle() != null) {
                customerNameBO.setString("prefixTitle", customer.getPrefixtitle());
            }

            // suffix
            if (customer.getSuffixtitle() != null) {
                customerNameBO.setString("suffixTitle", customer.getSuffixtitle());
            }
            
            customerBO.setDataObject("customerName",customerNameBO);
            	
            // DOB
            if (customer.getDateofbirth() != null) {
                customerBO.setDate("dateOfBirth", customer.getDateofbirth());
            }

            // country of citizenship
            if (customer.getCitizenshipcountry() != null) {
                customerBO.setString("citizenshipCountry", customer.getCitizenshipcountry());
            }

            // ssn
            if (customer.getSsn() != null) {
                customerBO.setString("ssn", customer.getSsn());
            }

            // customer status
            if (customer.getCustomerstatus() != null) {
                customerBO.setString("customerStatus", customer.getCustomerstatus());
            }

            // gender
            if (customer.getGender() != null) {
                customerBO.setString("gender", customer.getGender());
            }

            // marital status
            if (customer.getMaritalstatus() != null) {
                customerBO.setString("maritalStatus", customer.getMaritalstatus());
            }

            // new customer
            customerBO.setBoolean("newCustomer", false);

            // relationship code
            if (customer.getRelationshipcode() != null) {
                customerBO.setString("relationshipCode", customer.getRelationshipcode());
            }

            // relationship manager
            if (customer.getRelationshipmanager() != null) {
                customerBO.setString("relationshipManager", customer.getRelationshipmanager());
            }

            // residence status of us
            if (customer.getResidencestatusofus() != null) {
                customerBO.setString("residenceStatusOfUS", customer.getResidencestatusofus());
            }

            // postal address
            Postaladdress postaladdress = customer.getPostaladdressId();
            if (postaladdress != null) {
                DataObject postalAddressBO = createPostalAddress(postaladdress);

                customerBO.setDataObject("legalAddress", postalAddressBO);
            }

            // contact details
            Set<Contactdetails> contactDetailsSet = customer.getContactdetailsCollection();
            if (contactDetailsSet != null && !contactDetailsSet.isEmpty()) {
                // max one entry expected
                Contactdetails contactdetails = contactDetailsSet.iterator().next();
                DataObject contactDetailsBO = createContactDetailsBO(contactdetails);
                customerBO.setDataObject("contactDetails", contactDetailsBO);
            }

            // residence details
            Set<Residencedetails> residenceDetailsSet = customer.getResidencedetailsCollection();
            if (residenceDetailsSet != null && !residenceDetailsSet.isEmpty()) {
                // max one entry expected
                Residencedetails residencedetails = residenceDetailsSet.iterator().next();

                DataObject residenceDetailsBO = createResidenceDetailsBO(residencedetails);
                customerBO.setDataObject("residenceDetails", residenceDetailsBO);
            }

            // employment details
            Set<Employmentdetails> employmentDetailsSet = customer.getEmploymentdetailsCollection();
            if (employmentDetailsSet != null && !employmentDetailsSet.isEmpty()) {
                // max one entry expected
                Employmentdetails employmentdetails = employmentDetailsSet.iterator().next();

                DataObject employmenteDetailsBO = createEmploymentDetailsBO(employmentdetails);
                customerBO.setDataObject("employmentDetails", employmenteDetailsBO);
            }
        }
        return customerBO;
    }

    private DataObject createEmploymentDetailsBO(Employmentdetails employmentdetails) {
        DataObject employmentDetailsBO = IndustrySupportUtil.createDO(INVOLVED_PARTY_EXTN_NS, "EmploymentDetails");
        if (employmentDetailsBO != null) {

            // type
            if (employmentdetails.getType() != null) {
                employmentDetailsBO.setString("type", employmentdetails.getType());
            }

            // annual income
            employmentDetailsBO.setLong("annualIncome", employmentdetails.getAnnualincome());

            // employer name
            if (employmentdetails.getEmployername() != null) {
                employmentDetailsBO.setString("employerName", employmentdetails.getEmployername());
            }

            // position
            if (employmentdetails.getPosition() != null) {
                employmentDetailsBO.setString("position", employmentdetails.getPosition());
            }

            // home phone number
            if (employmentdetails.getHomephonenumber() != null) {
                employmentDetailsBO.setString("homePhoneNumber", employmentdetails.getHomephonenumber());
            }

            // mobile
            if (employmentdetails.getMobile() != null) {
                employmentDetailsBO.setString("mobile", employmentdetails.getMobile());
            }

            // postal address
            Postaladdress postaladdress = employmentdetails.getPostaladdressId();
            if (postaladdress != null) {
                DataObject legalAddress = createPostalAddress(postaladdress);

                employmentDetailsBO.setDataObject("legalAddress", legalAddress);
            }
        }
        return employmentDetailsBO;
    }

    private DataObject createPostalAddress(Postaladdress postaladdress) {
        DataObject postalAddressBO = IndustrySupportUtil.createDO(FINANCIAL_SVC_COMMON_NS, "PostalAddress");
        if (postaladdress != null) {

            List<DataObject> addressComponents = new ArrayList<DataObject>();

            // addressline1
            if (postaladdress.getAddressline1() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.ADDRESS_LINES,
                        postaladdress.getAddressline1()));
            }

            // addressline2
            if (postaladdress.getAddressline2() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.ADDRESS_LINES,
                        postaladdress.getAddressline2()));
            }

            // city
            if (postaladdress.getCity() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.CITY, postaladdress
                        .getCity()));
            }

            // region
            if (postaladdress.getState() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.REGION, postaladdress
                        .getState()));
            }

            // country
            if (postaladdress.getCountry() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.COUNTRY, postaladdress
                        .getCountry()));
            }

            // postal code
            if (postaladdress.getPostalcode() != null) {
                addressComponents.add(createAddressComponent(DataSetup.AddressComponentTypes.POSTAL_CODE, postaladdress
                        .getPostalcode()));
            }

            // setting components
            postalAddressBO.setList("addressComponents", addressComponents);
        }
        return postalAddressBO;
    }

    private DataObject createAddressComponent(String type, String value) {
        DataObject addressComponentBO = IndustrySupportUtil.createDO(FINANCIAL_SVC_COMMON_NS, "PostalAddressComponent");
        addressComponentBO.setString("type", type);
        addressComponentBO.setString("value", value);
        return addressComponentBO;
    }

    private DataObject createResidenceDetailsBO(Residencedetails residencedetails) {
        DataObject residenceDetailsBO = IndustrySupportUtil.createDO(INVOLVED_PARTY_EXTN_NS, "ResidenceDetails");
        if (residencedetails != null) {

            // country
            if (residencedetails.getCountry() != null) {
                residenceDetailsBO.setString("country", residencedetails.getCountry());
            }

            // years
            if (residencedetails.getYears() != null) {
                residenceDetailsBO.setString("years", residencedetails.getYears());
            }

            // status
            if (residencedetails.getStatus() != null) {
                residenceDetailsBO.setString("status", residencedetails.getStatus());
            }
        }
        return residenceDetailsBO;
    }

    private DataObject createContactDetailsBO(Contactdetails contactdetails) {
        DataObject contactDetailsBO = IndustrySupportUtil.createDO(INVOLVED_PARTY_EXTN_NS, "ContactDetails");
        if (contactdetails != null) {

            // email
            if (contactdetails.getEmail() != null) {
                contactDetailsBO.setString("email", contactdetails.getEmail());
            }

            // mobile
            if (contactdetails.getMobile() != null) {
                contactDetailsBO.setString("mobile", contactdetails.getMobile());
            }

            // home phone number
            if (contactdetails.getHomephonenumber() != null) {
                contactDetailsBO.setString("homePhoneNumber", contactdetails.getHomephonenumber());
            }

            // preferred language
            if (contactdetails.getPreferredlanguage() != null) {
                contactDetailsBO.setString("preferredLanguage", contactdetails.getPreferredlanguage());
            }

            // preferred medium
            if (contactdetails.getPreferredmedium() != null) {
                contactDetailsBO.setString("preferredMedium", contactdetails.getPreferredmedium());
            }

            // preferred time
            if (contactdetails.getPreferredtimeperiod() != null) {
                contactDetailsBO.setString("preferredTimePeriod", contactdetails.getPreferredtimeperiod());
            }
        }
        return contactDetailsBO;
    }

    private static IWicpDBManager wicpDBManager; 
    
    public static IWicpDBManager getNaoDBManager() {
    	if (wicpDBManager == null ) {
	        Properties dbProperties = new Properties();
	        dbProperties.setProperty(IWicpDBFactoryConstants.TYPE, "JPA");
	        dbProperties.setProperty(IWicpDBFactoryConstants.PERSISTENCE_UNIT_NAME, "new-account-opening-db");
	        wicpDBManager = WicpDBFactory.getDBManager(dbProperties);
    	}
        return wicpDBManager;
    }

    public static void main(String[] args) {
        NAOUtility utility = NAOUtility.getInstance();
        IWicpDBManager dbManager = NAOUtility.getNaoDBManager();

        Map<String, Object> paramMaps = new HashMap<String, Object>();
        paramMaps.put("loginid", "rverma");
        List<BaseObject> entities = dbManager.executeNamedQuery("findCustomerByLoginid", paramMaps);

        Customer customer = (Customer) entities.get(0);
        // utility.updateCustomer(customer, dbManager);
        utility.printCustomerEntity(customer);
        // IndustrySupportUtil.printBO(utility.createCustomerBO(customer));
    }

    @SuppressWarnings("unused")
    private void updateCustomer(Customer customer, IWicpDBManager dbManager) {
        customer.setDateofbirth(Date.valueOf("2010-09-06"));
        customer.setGender("Female");
        customer.setMaritalstatus("Married");
        customer.setSsn("123124343");
        customer.setResidencestatusofus("Temporary");
        customer.setRelationshipcode("100998");
        customer.setRelationshipmanager("Mukesh");
        customer.setCustomerstatus("New");
        customer.setPrefixtitle("Mrs");
        customer.setSuffixtitle("N/A");

        if (customer.getPostaladdressId() == null) {
            Postaladdress address = new Postaladdress();
            address.setAddressline1("E1-803, Jasminium");
            address.setAddressline2("Magarpattacity");
            address.setCity("Pune");
            address.setCountry("India");
            address.setState("Maharashtra");
            address.setPostalcode("400013");
            customer.setPostaladdressId(address);
            customer.setType("CUSTOMER");
        }
        Set<Employmentdetails> emplDtlsSet = customer.getEmploymentdetailsCollection();
        if (emplDtlsSet == null || emplDtlsSet.isEmpty()) {
            emplDtlsSet = new HashSet<Employmentdetails>();
        }

        dbManager.update(customer);
    }

    private void printCustomerEntity(Customer customer) {
        if (customer != null) {
            System.out.println(ReflectionToStringBuilder.toString(customer, ToStringStyle.MULTI_LINE_STYLE));
            System.out.println(ReflectionToStringBuilder.toString(customer.getPostaladdressId(),
                    ToStringStyle.MULTI_LINE_STYLE));
        }
    }
}
