/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Arrangement;
import com.ibm.websphere.icp.industry.db.entities.BaseObject;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class OpenAccountImpl {
    /**
     * Default constructor.
     */
    public OpenAccountImpl() {
        super();
    }

    private static final String OPEN_ACCOUNT_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/ArrangementSetup";

    private static final Log _log = LogFactory.getLog(OpenAccountImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "activateProductArrangement" defined for WSDL port type named
     * "ActivateProductArrangementPT".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject activateProductArrangement(DataObject aActivateProductArrangementRequest) {

        _log.info("Inside activateProductArrangement operation of OpenAccountImpl");

        DataObject responseBO = IndustrySupportUtil.createDO(OPEN_ACCOUNT_NS, "ActivateProductArrangementResponse");

        DataObject arrangementBO = aActivateProductArrangementRequest.getDataObject("arrangement");
        
        DataObject responseHeader = responseBO.createDataObject("responseHeader");

        if (arrangementBO != null) {
            String applicationNumber = arrangementBO.getString("applicationNumber");
            
            //System.out.println("application number: "+applicationNumber);

            if (StringUtils.isNotEmpty(applicationNumber)) {
                NAOUtility utility = NAOUtility.getInstance();
                IWicpDBManager dbManager = NAOUtility.getNaoDBManager();

                Map<String, Object> paramMaps = new HashMap<String, Object>();
                paramMaps.put("applicationnumber", applicationNumber);
                List<BaseObject> arrangements = dbManager.executeNamedQuery("findArrangementByApplicationNumber",
                        paramMaps);
                //System.out.println("arrangements size:"+arrangements.size());
                
                Arrangement arrangement = (Arrangement) arrangements.get(0);

                String accountNumber = null;

                Date today = Calendar.getInstance().getTime();

                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
                accountNumber = sdf.format(today);

                NumberFormat nf = NumberFormat.getInstance();
                nf.setMinimumIntegerDigits(6);
                nf.setMaximumFractionDigits(0);
                nf.setGroupingUsed(false);
                accountNumber = accountNumber + nf.format(arrangement.getArrangementId());
                //update arrangement status
                arrangement.setLifecyclestatus(DataSetup.LifecycleStatus.EFFECTIVE);
                
                if (StringUtils.isNotEmpty(accountNumber)) {
                    arrangement.setAccountnumber(accountNumber);
                    responseHeader.setString("id",accountNumber);

                    dbManager.update(arrangement);
                    
                }
            }
        }
        return responseBO;
    }

}