/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import commonj.sdo.DataObject;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Customer;
import com.ibm.websphere.icp.industry.db.entities.BaseObject;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.sca.ServiceManager;

public class RetrieveCustomerDetailsImpl {
    /**
     * Default constructor.
     */
    public RetrieveCustomerDetailsImpl() {
        super();
    }

    private static final String RETRIEVE_CUST_DTLS_NS = "http://www.ibm.com/industries/financialservices";

    private static final Log _log = LogFactory.getLog(RetrieveCustomerDetailsImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation
     * "retrieveCustomers" defined for WSDL port type named
     * "InvolvedPartyObjectManager".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject retrieveCustomers(DataObject aRetrieveCustomersRequest) {

        _log.info("Inside retrieveCustomers operation of RetrieveCustomerDetailsImpl");
        DataObject requestHeaderDO = aRetrieveCustomersRequest.getDataObject("requestHeader");
        String id = requestHeaderDO.getString("id");

        DataObject responseDO = IndustrySupportUtil.createDO(RETRIEVE_CUST_DTLS_NS, "RetrieveCustomersResponse");

        if (StringUtils.isNotEmpty(id)) {

            NAOUtility utility = NAOUtility.getInstance();
            IWicpDBManager dbManager = NAOUtility.getNaoDBManager();

            // get customer details
            Map<String, Object> params = new HashMap<String, Object>();

            // identifying type of query
            String[] identifiers = id.split(":");
            List<BaseObject> customers = null;
            if (identifiers[0].equals("loginId")) {
                params.put("loginid", identifiers[1]);
                customers = dbManager.executeNamedQuery("findCustomerByLoginid", params);
            } else {
                // default fall back
                params.put("customerId", Integer.parseInt(identifiers[1]));
                customers = dbManager.executeNamedQuery("findCustomerById", params);
            }
            // create customers list
            List<DataObject> customerBOList = new ArrayList<DataObject>();

            if (customers != null && !customers.isEmpty()) {
                // expecting only one entity
                Customer customer = (Customer) customers.get(0);

                // add customerDO
                customerBOList.add(utility.createCustomerBO(customer));
            }

            // set customers list
            responseDO.setList("customers", customerBOList);
        }

        return responseDO;
    }
}