/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.mock;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.banking.nao.entity.Customer;
import com.ibm.websphere.icp.industry.db.entities.BaseObject;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class SearchCustomerImpl {
    /**
     * Default constructor.
     */
    public SearchCustomerImpl() {
        super();
    }

    private static final String SEARCH_CUSTOMER_NS = "http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/SearchCustomer";

    private static final Log _log = LogFactory.getLog(SearchCustomerImpl.class);

    /**
     * Return a reference to the component service instance for this
     * implementation class. This method should be used when passing this
     * service to a partner reference or if you want to invoke this component
     * service asynchronously.
     * 
     * @generated (com.ibm.wbit.java)
     */
    @SuppressWarnings("unused")
    private Object getMyService() {
        return (Object) ServiceManager.INSTANCE.locateService("self");
    }

    /**
     * Method generated to support implementation of operation "searchCustomer"
     * defined for WSDL port type named "SearchCustomer".
     * 
     * The presence of commonj.sdo.DataObject as the return type and/or as a
     * parameter type conveys that it is a complex type. Please refer to the
     * WSDL Definition for more information on the type of input, output and
     * fault(s).
     */
    public DataObject searchCustomer(DataObject searchCustomerRequest) {

        _log.info("Inside searchCustomer operation of SearchCustomerImpl");

        DataObject searchCriteriaBO = searchCustomerRequest.getDataObject("searchCriteria");
        
        //IndustrySupportUtil.printBO(searchCriteriaBO);

        List<DataObject> customersList = new ArrayList<DataObject>();

        if (searchCriteriaBO != null) {

            NAOUtility utility = NAOUtility.getInstance();
            IWicpDBManager dbManger = NAOUtility.getNaoDBManager();

            // assumption is at least one criteria will be available
            String queryString = "SELECT c FROM Customer c ";

            StringBuilder sb = new StringBuilder();

            // search conditions list
            List<String> conditions = new ArrayList<String>();

            // query parameters map
            Map<String, Object> paramMap = new HashMap<String, Object>();

            String firstName = searchCriteriaBO.getString("firstName");

            String lastName = searchCriteriaBO.getString("lastName");

            String suffix = searchCriteriaBO.getString("suffix");

            String gender = searchCriteriaBO.getString("gender");

            Date dateOfBirth = searchCriteriaBO.getDate("dateOfBirth");

            String address1 = searchCriteriaBO.getString("address1");

            String address2 = searchCriteriaBO.getString("address2");

            String city = searchCriteriaBO.getString("city");

            String state = searchCriteriaBO.getString("state");

            String postalCode = searchCriteriaBO.getString("postalCode");

            // first name
            if (StringUtils.isNotEmpty(firstName)) {
                conditions.add("c.firstname LIKE :firstname");
                paramMap.put("firstname", "%" + firstName + "%");
            }

            // last name
            if (StringUtils.isNotEmpty(lastName)) {
                conditions.add("c.lastname LIKE :lastname");
                paramMap.put("lastname", "%" + lastName + "%");
            }

            // suffix
            if (StringUtils.isNotEmpty(suffix)) {
                conditions.add("c.suffixtitle LIKE :suffixtitle");
                paramMap.put("suffixtitle", "%" + suffix + "%");
            }

            // gender
            if (StringUtils.isNotEmpty(gender)) {
                conditions.add("c.gender = :gender");
                paramMap.put("gender", gender);
            }

            // date of birth
            if (dateOfBirth != null) {
                conditions.add("c.dateofbirth = :dateofbirth");
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                paramMap.put("dateofbirth", java.sql.Date.valueOf(sdf.format(dateOfBirth)));
            }

            // addressline1
            if (StringUtils.isNotEmpty(address1)) {
                conditions.add("c.postaladdressId.addressline1 LIKE :addressline1");
                paramMap.put("addressline1", "%" + address1 + "%");
            }

            // addressline2
            if (StringUtils.isNotEmpty(address2)) {
                conditions.add("c.postaladdressId.addressline2 LIKE :addressline2");
                paramMap.put("addressline2", "%" + address2 + "%");
            }

            // city
            if (StringUtils.isNotEmpty(city)) {
                conditions.add("c.postaladdressId.city LIKE :city");
                paramMap.put("city", "%" + city + "%");
            }

            // state
            if (StringUtils.isNotEmpty(state)) {
                conditions.add("c.postaladdressId.state LIKE :state");
                paramMap.put("state", "%" + state + "%");
            }

            // postal code
            if (StringUtils.isNotEmpty(postalCode)) {
                conditions.add("c.postaladdressId.postalcode LIKE :postalcode");
                paramMap.put("postalcode", "%" + postalCode + "%");
            }

            int size = conditions.size();
            for (int i = 0; i < size; i++) {
                if (i == 0) {
                    sb.append(" WHERE ");
                } else {
                    sb.append(" AND ");
                }
                sb.append(conditions.get(i));
            }

            // final query
            queryString = queryString + sb.toString();

            _log.info("final query string:" + queryString);

            List<BaseObject> customers = dbManger.executeDynamicQuery(queryString, paramMap);

            if (customers != null && !customers.isEmpty()) {
                for (BaseObject customer : customers) {
                    customersList.add(utility.createCustomerBO((Customer) customer));
                }
            }

        }

        DataObject responseDO = IndustrySupportUtil.createDO(SEARCH_CUSTOMER_NS, "SearchCustomerResponse");
        responseDO.setList("customers", customersList);
        return responseDO;
    }

}