/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package documentchecklist;




import ilog.connector.ibm.runtime.DecisionService;
import ilog.connector.ibm.runtime.DecisionServiceHelper;
import ilog.connector.ibm.runtime.exception.DecisionServiceException;

import ilog.rules.res.model.IlrPath;
import ilog.rules.res.session.IlrPOJOSessionFactory;
import ilog.rules.res.session.IlrSessionFactory;
import ilog.rules.res.session.IlrSessionRequest;
import ilog.rules.res.session.IlrSessionResponse;
import ilog.rules.res.session.IlrStatelessSession;
import commonj.sdo.DataObject;

import java.util.*;

/**
 * This class contains one method for each ruleset of the RuleApp to run. 
 */
public class NAORuleDecisionService extends DecisionService {

	/**
	 * The name of the RuleApp.
	 */
	public static final String RULEAPP_NAME = "NAORuleApp";

	/**
	 * The name of the NAORule Ruleset.
	 */
	public static final String RULESET_NAME_NAORULE = "NAORule";

	/**
	 * Invokes NAORule ruleset.
	 */
	public DataObject NAORule(DataObject requestSdo)
			throws DecisionServiceException {
		String rulesetPath = "/" + RULEAPP_NAME + "/" + RULESET_NAME_NAORULE;
		try {
			//Sets IN and INOUT parameters.
			Map inputParameters = new HashMap();
			String strRequest = serialize(
					requestSdo,
					"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema/DocumentCheckList",
					"DocumentCheckListRequest");
			inputParameters.put("request", strRequest);

			//Get the session factory for a Java rulesession invocation
			IlrSessionFactory sessionFactory = new IlrPOJOSessionFactory();
			IlrSessionRequest request = sessionFactory.createRequest();
			request.setRulesetPath(IlrPath.parsePath(rulesetPath));
			request.setExecutionId("ID:" + System.currentTimeMillis());
			request.setTraceEnabled(true);
			request.getTraceFilter().setInfoAllFilters(true);
			request.getInputParameters().putAll(inputParameters);

			//create a stateless rule session
			IlrStatelessSession ruleSession = sessionFactory
					.createStatelessSession();
			IlrSessionResponse result = ruleSession.execute(request);
			Map outputParameters = result.getOutputParameters();

			String responseParamVariable_ = (String) outputParameters
					.get("response");

			DataObject response = createResponse("http://documentchecklist",
					"NAORuleExecutionResult");

			response.set("response", deserialize(responseParamVariable_,
					"DocumentCheckListResponse"));

			return response;
		} catch (Exception e) {
			String detailStackTrace = DecisionServiceHelper.format(e);
			DecisionServiceException dsException = new DecisionServiceException(
					e.getMessage(), "Failed to invoke decision service: "
							+ rulesetPath, detailStackTrace);
			throw dsException;
		}
	}

}
