/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
/**
 * Copyright IBM Corp. 2009, 2010
 */
package com.ibm.websphere.ilog.nao.test;

import ilog.rules.archive.IlrJarArchiveLoader;
import ilog.rules.archive.IlrRulesetArchiveLoader;
import ilog.rules.engine.IlrContext;
import ilog.rules.engine.IlrParameterMap;
import ilog.rules.engine.IlrRuleset;
import ilog.rules.engine.IlrRulesetArchiveParser;
import ilog.rules.factory.IlrReflect;
import ilog.rules.xml.IlrXmlErrorException;
import ilog.rules.xml.IlrXmlObject;
import ilog.rules.xml.binding.IlrXmlDefaultDataDriver;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.jar.JarInputStream;

public class JRulesRunner {

    public static void executeEngine(String rulesetPath, String requestXMLFile, String responseXMLFile)
            throws Exception {
        JarInputStream is = new JarInputStream(new FileInputStream(new File(rulesetPath)));
        IlrRulesetArchiveLoader rulesetloader = new IlrJarArchiveLoader(is);
        IlrRulesetArchiveParser rulesetparser = new IlrRulesetArchiveParser();
        IlrRuleset ruleset = new IlrRuleset();
        rulesetparser.setRuleset(ruleset);
        boolean parsed = rulesetparser.parseArchive(rulesetloader);
        if (!parsed) {
            System.err.println("Ruleset is not valid!");
            return;
        }

        IlrContext context = new IlrContext(ruleset);
        IlrParameterMap inputs = new IlrParameterMap();
        inputs.setParameter("request", getRequest(requestXMLFile, context));
        context.setParameters(inputs);

        IlrParameterMap outputs = context.execute();
        IlrXmlObject response = (IlrXmlObject) outputs.getObjectValue("response");
        System.out.println("Fired rules = " + outputs.getIntValue("ilog.rules.firedRulesCount"));

        writeResponse(responseXMLFile, context, response);

        context.retractAll();
        context.end();
    }

    private static void writeResponse(String responseXMLFile, IlrContext context, IlrXmlObject response)
            throws IlrXmlErrorException, IOException {
        IlrReflect reflect = context.getRuleset().getReflect();
        IlrXmlDefaultDataDriver driver = new IlrXmlDefaultDataDriver(reflect);
        Writer writer = new FileWriter(responseXMLFile);
        driver.writeObject(response, writer);
    }

    private static IlrXmlObject getRequest(String requestXMLFile, IlrContext context) throws IlrXmlErrorException,
            FileNotFoundException {
        IlrReflect reflect = context.getRuleset().getReflect();
        IlrXmlDefaultDataDriver driver = new IlrXmlDefaultDataDriver(reflect);
        IlrXmlObject request = driver.readObject(new FileReader(requestXMLFile));
        return request;
    }

}
