/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.entity;

import java.io.Serializable;
import java.sql.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.NamedQuery;
import javax.persistence.NamedQueries;

@Entity
@NamedQueries( {
        @NamedQuery(name = "findArrangementById", query = "SELECT a FROM Arrangement a WHERE a.arrangementId = :arrangementId"),
        @NamedQuery(name = "findArrangementByApplicationNumber", query = "SELECT a FROM Arrangement a WHERE a.applicationnumber = :applicationnumber") })
public class Arrangement extends BankingBaseObject implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "ARRANGEMENT_ID")
    private int arrangementId;

    private String description;

    private Date enddate;

    private String name;

    private Date startdate;

    private String lifecyclestatus;

    private String lifecyclephaseinitiation;

    private String applicationnumber;

    private String currency;

    private String accountnumber;

    private String accounttype;

    private String product;

    private String card;

    private String internetbanking;

    private String ivr;

    private String checkbook;

    private int checkquantity;

    private String operatinginstruction;

    private String approvedby;

    private Date depositdate;

    private String statementtype;

    private String statementfrequency;

    @ManyToOne(cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    @JoinColumn(name = "CUSTOMER_ID")
    private Customer customerId;

    @OneToMany(mappedBy = "arrangementId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Term> termCollection;

    @OneToMany(mappedBy = "arrangementId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Arrangementdocuments> arrangementdocumentsCollection;

    @ManyToMany(cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    @JoinTable(name = "COAPPLICANTS", joinColumns = @JoinColumn(name = "ARRANGEMENT_ID"), inverseJoinColumns = @JoinColumn(name = "CUSTOMER_ID"))
    private Set<Customer> customerCollection;

    private static final long serialVersionUID = 1L;

    public Arrangement() {
        super();
    }

    public int getArrangementId() {
        return this.arrangementId;
    }

    public void setArrangementId(int arrangementId) {
        this.arrangementId = arrangementId;
    }

    public String getDescription() {
        return (this.description != null) ? this.description.trim() : this.description;
    }

    public void setDescription(String description) {
        this.description = (description != null) ? description.trim() : description;
    }

    public Date getEnddate() {
        return this.enddate;
    }

    public void setEnddate(Date enddate) {
        this.enddate = enddate;
    }

    public String getName() {
        return (this.name != null) ? this.name.trim() : this.name;
    }

    public void setName(String name) {
        this.name = (name != null) ? name.trim() : name;
    }

    public Date getStartdate() {
        return this.startdate;
    }

    public void setStartdate(Date startdate) {
        this.startdate = startdate;
    }

    public String getLifecyclestatus() {
        return (this.lifecyclestatus!= null) ? this.lifecyclestatus.trim() : this.lifecyclestatus;
    }

    public void setLifecyclestatus(String lifecyclestatus) {
        this.lifecyclestatus = (lifecyclestatus!= null) ? lifecyclestatus.trim() : lifecyclestatus;
    }

    public String getLifecyclephaseinitiation() {
        return (this.lifecyclephaseinitiation != null) ? this.lifecyclephaseinitiation.trim() : this.lifecyclephaseinitiation;
    }

    public void setLifecyclephaseinitiation(String lifecyclephaseinitiation) {
        this.lifecyclephaseinitiation = (lifecyclephaseinitiation!= null) ? lifecyclephaseinitiation.trim() : lifecyclephaseinitiation;
    }

    public String getApplicationnumber() {
        return (this.applicationnumber!= null) ? this.applicationnumber.trim() : this.applicationnumber;
    }

    public void setApplicationnumber(String applicationnumber) {
        this.applicationnumber = (applicationnumber != null) ? applicationnumber.trim() : applicationnumber;
    }

    public String getCurrency() {
        return (this.currency != null) ? this.currency.trim() : this.currency;
    }

    public void setCurrency(String currency) {
        this.currency = (currency != null) ? currency.trim() : currency;
    }

    public String getAccountnumber() {
        return (this.accountnumber != null) ? this.accountnumber.trim() : this.accountnumber;
    }

    public void setAccountnumber(String accountnumber) {
        this.accountnumber = (accountnumber != null) ? accountnumber.trim() : accountnumber;
    }

    public String getAccounttype() {
        return (this.accounttype != null) ? this.accounttype.trim() : this.accounttype;
    }

    public void setAccounttype(String accounttype) {
        this.accounttype = (accounttype != null) ? accounttype.trim() : accounttype;
    }

    public String getProduct() {
        return (this.product != null) ? this.product.trim() : this.product;
    }

    public void setProduct(String product) {
        this.product = (product != null) ? product.trim() : product;
    }

    public String getCard() {
        return (this.card != null) ? this.card.trim() : this.card;
    }

    public void setCard(String card) {
        this.card = (card != null) ? card.trim() : card;
    }

    public String getInternetbanking() {
        return (this.internetbanking != null) ? this.internetbanking.trim() : this.internetbanking;
    }

    public void setInternetbanking(String internetbanking) {
        this.internetbanking = (internetbanking != null) ? internetbanking.trim() : internetbanking;
    }

    public String getIvr() {
        return (this.ivr != null) ? this.ivr.trim() : this.ivr;
    }

    public void setIvr(String ivr) {
        this.ivr = (ivr != null) ? ivr.trim() : ivr;
    }

    public String getCheckbook() {
        return (this.checkbook != null) ? this.checkbook.trim() : this.checkbook;
    }

    public void setCheckbook(String checkbook) {
        this.checkbook = (checkbook != null) ? checkbook.trim() : checkbook;
    }

    public int getCheckquantity() {
        return this.checkquantity;
    }

    public void setCheckquantity(int checkquantity) {
        this.checkquantity = checkquantity;
    }

    public String getOperatinginstruction() {
        return (this.operatinginstruction != null) ? this.operatinginstruction.trim() : this.operatinginstruction;
    }

    public void setOperatinginstruction(String operatinginstruction) {
        this.operatinginstruction = (operatinginstruction != null) ? operatinginstruction.trim() : operatinginstruction;
    }

    public String getApprovedby() {
        return (this.approvedby != null) ? this.approvedby.trim() : this.approvedby;
    }

    public void setApprovedby(String approvedby) {
        this.approvedby = (approvedby != null) ? approvedby.trim() : approvedby;
    }

    public Date getDepositdate() {
        return this.depositdate;
    }

    public void setDepositdate(Date depositdate) {
        this.depositdate = depositdate;
    }

    public String getStatementtype() {
        return (this.statementtype != null) ? this.statementtype.trim() : this.statementtype;
    }

    public void setStatementtype(String statementtype) {
        this.statementtype = (statementtype != null) ? statementtype.trim() : statementtype;
    }

    public String getStatementfrequency() {
        return (this.statementfrequency != null) ? this.statementfrequency.trim() : this.statementfrequency;
    }

    public void setStatementfrequency(String statementfrequency) {
        this.statementfrequency = (statementfrequency != null) ? statementfrequency.trim() : statementfrequency;
    }

    public Customer getCustomerId() {
        return this.customerId;
    }

    public void setCustomerId(Customer customerId) {
        this.customerId = customerId;
    }

    public Set<Term> getTermCollection() {
        return this.termCollection;
    }

    public void setTermCollection(Set<Term> termCollection) {
        this.termCollection = termCollection;
    }

    public Set<Arrangementdocuments> getArrangementdocumentsCollection() {
        return this.arrangementdocumentsCollection;
    }

    public void setArrangementdocumentsCollection(Set<Arrangementdocuments> arrangementdocumentsCollection) {
        this.arrangementdocumentsCollection = arrangementdocumentsCollection;
    }

    public Set<Customer> getCustomerCollection() {
        return this.customerCollection;
    }

    public void setCustomerCollection(Set<Customer> customerCollection) {
        this.customerCollection = customerCollection;
    }

}
