/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.nao.entity;

import java.io.Serializable;
import java.sql.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

@Entity
@NamedQueries( {
        @NamedQuery(name = "findCustomerById", query = "SELECT c FROM Customer c WHERE c.customerId = :customerId"),
        @NamedQuery(name = "findCustomerByLoginid", query = "SELECT c FROM Customer c WHERE c.loginid = :loginid") })
public class Customer extends BankingBaseObject implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "CUSTOMER_ID")
    private int customerId;

    private Date dateofbirth;

    private String gender;

    private String maritalstatus;

    private String ssn;

    private int age;

    private String citizenshipcountry;

    private String residencestatusofus;

    private String relationshipmanager;

    private String relationshipcode;

    private String customerstatus;

    private String newcustomer;

    private String loginid;

    private String description;

    private Date enddate;

    private String nametext;

    private Date startdate;

    private String type;

    private String firstname;

    private String lastname;

    private String middlenames;

    private String prefixtitle;

    private String shortfirstname;

    private String suffixtitle;

    @ManyToOne(cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    @JoinColumn(name = "POSTALADDRESS_ID")
    private Postaladdress postaladdressId;

    @OneToMany(mappedBy = "customerId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Contactdetails> contactdetailsCollection;

    @OneToMany(mappedBy = "customerId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Residencedetails> residencedetailsCollection;

    @OneToMany(mappedBy = "customerId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Employmentdetails> employmentdetailsCollection;

    @OneToMany(mappedBy = "customerId", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Arrangement> arrangementCollection;

    @ManyToMany(mappedBy = "customerCollection", cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
    private Set<Arrangement> arrangementCollection2;

    private static final long serialVersionUID = 1L;

    public Customer() {
        super();
    }

    public int getCustomerId() {
        return this.customerId;
    }

    public void setCustomerId(int customerId) {
        this.customerId = customerId;
    }

    public Date getDateofbirth() {
        return this.dateofbirth;
    }

    public void setDateofbirth(Date dateofbirth) {
        this.dateofbirth = dateofbirth;
    }

    public String getGender() {
        return (this.gender != null) ? this.gender.trim() : this.gender;
    }

    public void setGender(String gender) {
        this.gender = (gender != null) ? gender.trim() : gender;
    }

    public String getMaritalstatus() {
        return (this.maritalstatus != null) ? this.maritalstatus.trim() : this.maritalstatus;
    }

    public void setMaritalstatus(String maritalstatus) {
        this.maritalstatus = (maritalstatus != null) ? maritalstatus.trim() : maritalstatus;
    }

    public String getSsn() {
        return (this.ssn != null) ? this.ssn.trim() : this.ssn;
    }

    public void setSsn(String ssn) {
        this.ssn = (ssn != null) ? ssn.trim() : ssn;
    }

    public int getAge() {
        return this.age;
    }

    public void setAge(int age) {
        this.age = age;
    }

    public String getCitizenshipcountry() {
        return (this.citizenshipcountry != null) ? this.citizenshipcountry.trim() : this.citizenshipcountry;
    }

    public void setCitizenshipcountry(String citizenshipcountry) {
        this.citizenshipcountry = (citizenshipcountry != null) ? citizenshipcountry.trim() : citizenshipcountry;
    }

    public String getResidencestatusofus() {
        return (this.residencestatusofus != null) ? this.residencestatusofus.trim() : this.residencestatusofus;
    }

    public void setResidencestatusofus(String residencestatusofus) {
        this.residencestatusofus = (residencestatusofus != null) ? residencestatusofus.trim() : residencestatusofus;
    }

    public String getRelationshipmanager() {
        return (this.relationshipmanager != null) ? this.relationshipmanager.trim() : this.relationshipmanager;
    }

    public void setRelationshipmanager(String relationshipmanager) {
        this.relationshipmanager = (relationshipmanager != null) ? relationshipmanager.trim() : relationshipmanager;
    }

    public String getRelationshipcode() {
        return (this.relationshipcode != null) ? this.relationshipcode.trim() : this.relationshipcode;
    }

    public void setRelationshipcode(String relationshipcode) {
        this.relationshipcode = (relationshipcode!= null) ? relationshipcode.trim() : relationshipcode;
    }

    public String getCustomerstatus() {
        return (this.customerstatus != null) ? this.customerstatus.trim() : this.customerstatus;
    }

    public void setCustomerstatus(String customerstatus) {
        this.customerstatus = (customerstatus != null) ? customerstatus.trim() : customerstatus;
    }

    public String getNewcustomer() {
        return (this.newcustomer != null) ? this.newcustomer.trim() : this.newcustomer;
    }

    public void setNewcustomer(String newcustomer) {
        this.newcustomer = (newcustomer != null) ? newcustomer.trim() : newcustomer;
    }

    public String getLoginid() {
        return (this.loginid != null) ? this.loginid.trim() : this.loginid;
    }

    public void setLoginid(String loginid) {
        this.loginid = (loginid != null) ? loginid.trim() : loginid;
    }

    public String getDescription() {
        return (this.description != null) ? this.description.trim() : this.description;
    }

    public void setDescription(String description) {
        this.description = (description != null) ? description.trim() : description;
    }

    public Date getEnddate() {
        return this.enddate;
    }

    public void setEnddate(Date enddate) {
        this.enddate = enddate;
    }

    public String getNametext() {
        return (this.nametext != null) ? this.nametext.trim() : this.nametext;
    }

    public void setNametext(String nametext) {
        this.nametext = (nametext != null) ? nametext.trim() : nametext;
    }

    public Date getStartdate() {
        return this.startdate;
    }

    public void setStartdate(Date startdate) {
        this.startdate = startdate;
    }

    public String getType() {
        return (this.type != null) ? this.type.trim() : this.type;
    }

    public void setType(String type) {
        this.type = (type != null) ? type.trim() : type;
    }

    public String getFirstname() {
        return (this.firstname != null) ? this.firstname.trim() : this.firstname;
    }

    public void setFirstname(String firstname) {
        this.firstname = (firstname != null) ? firstname.trim() : firstname;
    }

    public String getLastname() {
        return (this.lastname != null) ? this.lastname.trim() : this.lastname;
    }

    public void setLastname(String lastname) {
        this.lastname = (lastname != null) ? lastname.trim() : lastname;
    }

    public String getMiddlenames() {
        return (this.middlenames != null) ? this.middlenames.trim() : this.middlenames;
    }

    public void setMiddlenames(String middlenames) {
        this.middlenames = (middlenames != null) ? middlenames.trim() : middlenames;
    }

    public String getPrefixtitle() {
        return (this.prefixtitle != null) ? this.prefixtitle.trim() : this.prefixtitle;
    }

    public void setPrefixtitle(String prefixtitle) {
        this.prefixtitle = (prefixtitle != null) ? prefixtitle.trim() : prefixtitle;
    }

    public String getShortfirstname() {
        return (this.shortfirstname != null) ? this.shortfirstname.trim() : this.shortfirstname;
    }

    public void setShortfirstname(String shortfirstname) {
        this.shortfirstname = (shortfirstname != null) ? shortfirstname.trim() : shortfirstname;
    }

    public String getSuffixtitle() {
        return (this.suffixtitle != null) ? this.suffixtitle.trim() : this.suffixtitle;
    }

    public void setSuffixtitle(String suffixtitle) {
        this.suffixtitle = (suffixtitle != null) ? suffixtitle.trim() : suffixtitle;
    }

    public Postaladdress getPostaladdressId() {
        return this.postaladdressId;
    }

    public void setPostaladdressId(Postaladdress postaladdressId) {
        this.postaladdressId = postaladdressId;
    }

    public Set<Contactdetails> getContactdetailsCollection() {
        return this.contactdetailsCollection;
    }

    public void setContactdetailsCollection(Set<Contactdetails> contactdetailsCollection) {
        this.contactdetailsCollection = contactdetailsCollection;
    }

    public Set<Residencedetails> getResidencedetailsCollection() {
        return this.residencedetailsCollection;
    }

    public void setResidencedetailsCollection(Set<Residencedetails> residencedetailsCollection) {
        this.residencedetailsCollection = residencedetailsCollection;
    }

    public Set<Employmentdetails> getEmploymentdetailsCollection() {
        return this.employmentdetailsCollection;
    }

    public void setEmploymentdetailsCollection(Set<Employmentdetails> employmentdetailsCollection) {
        this.employmentdetailsCollection = employmentdetailsCollection;
    }

    public Set<Arrangement> getArrangementCollection() {
        return this.arrangementCollection;
    }

    public void setArrangementCollection(Set<Arrangement> arrangementCollection) {
        this.arrangementCollection = arrangementCollection;
    }

    public Set<Arrangement> getArrangementCollection2() {
        return this.arrangementCollection2;
    }

    public void setArrangementCollection2(Set<Arrangement> arrangementCollection2) {
        this.arrangementCollection2 = arrangementCollection2;
    }

}
