/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.custom.map;


import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import com.ibm.websphere.sca.sdo.DataFactory;
import commonj.sdo.DataObject;

public class CustomAccountToProductArrangementDataObjectMapper {

	private CustomAccountToProductArrangementDataObjectMapper() {

	}

	public static void customMapFromAccountToProductArrangementExtension(
			DataObject srcObj, DataObject productArrangementExtn)
			throws MapException {

		if (srcObj == null || productArrangementExtn == null) {
			throw new MapException("Source or target can not be Null Target ");
		}

		DataObject customerSrc = srcObj.getDataObject("customer");

		DataObject customerExtn = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty",
						"CustomerExtn");

		productArrangementExtn.setDataObject("customer", customerExtn);

		if (customerSrc != null) {
			// mapping Customer To Product Arrangement
			mapCustomerToProductArrangementExtn(customerSrc, customerExtn);
		}

		DataObject accountDetails = srcObj.getDataObject("accountDetails");

		if (accountDetails != null) {

			customMapFromAccountDetailsToProductArrangementExtension(
					accountDetails, productArrangementExtn);
		}
	
		mapDocumentToProductArrangementExtn(srcObj,productArrangementExtn);
		
		mapRemainingAccountObjectToProductArrangementExtension(srcObj,
				productArrangementExtn);
	}

	public static void mapDocumentToProductArrangementExtn(
			DataObject account, DataObject productArrangementExtn) {
		
		DataObject document = account.getDataObject("document");
		
		if(document != null) {
		boolean payStubs = document.getBoolean("payStubs");
		boolean w2form = document.getBoolean("w2form");
		boolean twoYearsIncomeTaxReturns = document.getBoolean("twoYearsIncomeTaxReturns");
		boolean taxReturn = document.getBoolean("taxReturn");
		boolean ssnCard = document.getBoolean("ssnCard");
		boolean drivingLicense = document.getBoolean("drivingLicense");
		boolean visaInfo = document.getBoolean("visaInfo");
		boolean poc = document.getBoolean("proofOfCitizenship");
		
		List documents = new ArrayList<String>();
		
		if(payStubs) {
			documents.add("Pay stubs");
		}
		
		if(w2form) {
			documents.add("W2 form");
		}
 		
		if(twoYearsIncomeTaxReturns) {
			documents.add("Two years of income tax returns");
		}
		
		if(taxReturn) {
			documents.add("Tax return");
		}
		
		if(ssnCard) {
			documents.add("SSN Card");
		}
		
		if(drivingLicense) {
			documents.add("Driving License");
		}
		
		if(poc) {
			documents.add("Proof of Citizenship/PR");
		}
		
		if(visaInfo) {
			documents.add("Visa Information");
		}
	
		productArrangementExtn.setList("documents",documents);
		}
	}

	private static void mapRemainingAccountObjectToProductArrangementExtension(
			DataObject accountSrc, DataObject productArrangmentExtn) {

		String operatingInstructions = accountSrc
				.getString("operatingInstructions");
		String accountStatement = accountSrc.getString("accountStatement");
		String subscriptionType = accountSrc.getString("subscriptionType");
		String card = accountSrc.getString("card");
		String internetBanking = accountSrc.getString("internetBanking");
		String ivr = accountSrc.getString("IVR");
		String checks = accountSrc.getString("checks");
		int quantity = accountSrc.getInt("quantity");
		String applicationno = accountSrc.getString("applicationno");

		if (operatingInstructions != null) {
			productArrangmentExtn.setString("operatingInstruction",
					operatingInstructions);
		}
		if(applicationno != null) {
			productArrangmentExtn.setString("applicationNumber",applicationno);
		}
		if (accountStatement != null && subscriptionType != null) {
			DataObject accountStatementDO = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/Arrangement",
							"AccountStatement");
			accountStatementDO.setString("type", accountStatement);
			accountStatementDO.setString("frequency", subscriptionType);
			productArrangmentExtn
					.setDataObject("statement", accountStatementDO);
		}
		if (card != null) {
			productArrangmentExtn.setString("card", card);
		}

		if (internetBanking != null) {
			if(internetBanking.equalsIgnoreCase("Yes")) {
				productArrangmentExtn.setBoolean("internetBanking", true);
			} else {
				productArrangmentExtn.setBoolean("internetBanking", false);
			}
			
		}
		if (ivr != null) {
			if(ivr.equalsIgnoreCase("Yes")) {
				productArrangmentExtn.setBoolean("ivr", true);
			} else {
				productArrangmentExtn.setBoolean("ivr", false);
			}
			
		}
		if (checks != null) {
			if(checks.equals("Yes")) {
				productArrangmentExtn.setBoolean("checkBook", true);
				if (new Integer(quantity) != null || quantity > 0) {
					productArrangmentExtn.setInt("checkQuantity", quantity);
				}
			} else {
				productArrangmentExtn.setBoolean("checkBook", false);
			}
		}

		

		DataObject coApplicantInformation = accountSrc
				.getDataObject("coApplicantInformation");
		if (coApplicantInformation != null) {
			String coapplicant = coApplicantInformation
					.getString("coApplicant");
			if (coapplicant != null) {
				if(coapplicant.equals("Yes")) {
				List coApplicantInformationList = coApplicantInformation
						.getList("coApplicantList");
				if (coApplicantInformationList != null) {
					if (!coApplicantInformationList.isEmpty()) {
						List coApplicantList = new ArrayList();
						for (Iterator iterator = coApplicantInformationList
								.iterator(); iterator.hasNext();) {
							DataObject coApplicants = (DataObject) iterator
									.next();
							DataObject coApplicantInfo = DataFactory.INSTANCE
									.create(
											"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty",
											"CustomerExtn");
							DataObject coapplicantName = DataFactory.INSTANCE
									.create(
											"http://www.ibm.com/industries/financialservices",
											"IndividualName");
							String name = coApplicants.getString("name");
							if (name != null) {
								coapplicantName.setString("firstName", name);
								coApplicantInfo.setDataObject("customerName",
										coapplicantName);
							}
							int age = coApplicants.getInt("age");
							if (age > 0 || new Integer(age) != null) {
								coApplicantInfo.setInt("age", age);
							}
							String address = coApplicants.getString("address");
							if (address != null) {
								DataObject postalAddress = DataFactory.INSTANCE
										.create(
												"http://www.ibm.com/industries/financialservices",
												"PostalAddress");
								List addressComponentList = new ArrayList();
								DataObject addressComponent = DataFactory.INSTANCE
										.create(
												"http://www.ibm.com/industries/financialservices",
												"PostalAddressComponent");
								addressComponent.setString("value", address);
								addressComponent.setString("type", "addressLines");
								addressComponentList.add(addressComponent);
								postalAddress.setList("addressComponents",
										addressComponentList);
								coApplicantInfo.setDataObject("legalAddress",
										postalAddress);
							}
							coApplicantList.add(coApplicantInfo);

						}
						productArrangmentExtn.setList("coApplicants",
								coApplicantList);
					}

				}
			}
		}
		}

	}

	public static void mapCustomerToProductArrangementExtn(
			DataObject customer, DataObject customerExtn) {
		DataObject customerDetailsSrc = customer
				.getDataObject("customerDetails");

		if (customerDetailsSrc != null) {
			// mapping customerDetails to CustomerExtension
			customMapFromCustomerDetailsToCustomerExtension(customerDetailsSrc,
					customerExtn);
		}
		DataObject residenceDetailsSrc = customer
				.getDataObject("residenceDetails");

		if (residenceDetailsSrc != null) {
			customMapFromResidenceDetailsToCustomerExtension(
					residenceDetailsSrc, customerExtn);
		}

		DataObject contactDetails = customer.getDataObject("contactDetails");

		if (contactDetails != null) {
			customMapFromContactDetailsToCustomerExtension(contactDetails,
					customerExtn);
		}

		DataObject employmentDetails = customer
				.getDataObject("employmentDetails");

		if (employmentDetails != null) {
			customMapFromEmploymentDetailsToCustomerExtension(
					employmentDetails, customerExtn);
		}

		DataObject relationshipDetails = customer
				.getDataObject("relationshipDetails");

		if (relationshipDetails != null) {
			customMapFromRelationshipDetailsToCustomerExtension(
					relationshipDetails, customerExtn);
		}
	}

	private static void customMapFromRelationshipDetailsToCustomerExtension(
			DataObject relationshipDetailsSrc, DataObject customerExtn) {
		String relationshipManager = relationshipDetailsSrc.getString("relationshipManager");
		String relationshipCode = relationshipDetailsSrc.getString("relationshipCode");

		if (relationshipManager != null) {
			customerExtn.setString("relationshipManager", relationshipManager);
		}
		if (relationshipCode != null) {
			customerExtn.setString("relationshipCode", relationshipCode);
		}
	}

	private static void customMapFromEmploymentDetailsToCustomerExtension(
			DataObject employmentDetailsSrc, DataObject customerExtn) {

		String employement = employmentDetailsSrc.getString("employement");
		long annualIncome = employmentDetailsSrc.getLong("annualIncome");
		String name = employmentDetailsSrc.getString("name");
		String faddress = employmentDetailsSrc.getString("faddress");
		String saddress = employmentDetailsSrc.getString("saddress");
		String city = employmentDetailsSrc.getString("city");
		String position = employmentDetailsSrc.getString("position");
		String landline = employmentDetailsSrc.getString("landline");
		String state = employmentDetailsSrc.getString("state");
		String mobile = employmentDetailsSrc.getString("mobile");
		String zip = employmentDetailsSrc.getString("zip");

		DataObject employmentDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty",
						"EmploymentDetails");
		customerExtn.setDataObject("employmentDetails", employmentDetails);

		if (employement != null) {
			employmentDetails.setString("type", employement);
		}

		if (new Long(annualIncome) != null) {
			employmentDetails.setLong("annualIncome", annualIncome);
		}

		if (name != null) {
			employmentDetails.setString("employerName", name);
		}

		if (position != null) {
			employmentDetails.setString("position", position);
		}

		if (landline != null) {
			employmentDetails.setString("homePhoneNumber", landline);
		}

		if (mobile != null) {
			employmentDetails.setString("mobile", mobile);
		}

		if (faddress != null) {

			DataObject postalAddress = DataFactory.INSTANCE.create(
					"http://www.ibm.com/industries/financialservices",
					"PostalAddress");
			employmentDetails.setDataObject("legalAddress", postalAddress);
			DataObject postalAddressComponent1 = DataFactory.INSTANCE.create(
					"http://www.ibm.com/industries/financialservices",
					"PostalAddressComponent");
			List addressComponentsList = new ArrayList<DataObject>();
			addressComponentsList.add(postalAddressComponent1);

			postalAddressComponent1.setString("value", faddress);
			postalAddressComponent1.setString("type", "addressLines");

			if (saddress != null) {
				//System.out.println("inside saddress null");
				DataObject postalAddressComponent2 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent2.setString("value", saddress);
				postalAddressComponent2.setString("type", "addressLines");
				addressComponentsList.add(postalAddressComponent2);
			}
			if (city != null) {
				DataObject postalAddressComponent3 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent3.setString("value", city);
				postalAddressComponent3.setString("type", "city");
				addressComponentsList.add(postalAddressComponent3);
			}

			if (state != null) {
				DataObject postalAddressComponent4 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent4.setString("value", state);
				postalAddressComponent4.setString("type", "region");
				addressComponentsList.add(postalAddressComponent4);
			}

			if (zip != null) {
				DataObject postalAddressComponent5 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent5.setString("value", zip);
				postalAddressComponent5.setString("type", "postalCode");
				addressComponentsList.add(postalAddressComponent5);
			}
			postalAddress.setList("addressComponents", addressComponentsList);
		}
	}

	private static void customMapFromContactDetailsToCustomerExtension(
			DataObject contactDetailsSrc, DataObject customerExtn) {

		String planguage = contactDetailsSrc.getString("preferredLanguage");
		String landline = contactDetailsSrc.getString("landline");
		String mobile = contactDetailsSrc.getString("mobile");
		String emailId = contactDetailsSrc.getString("emailId");
		String contactWhen = contactDetailsSrc.getString("contactWhen");
		String contactPreference = contactDetailsSrc
				.getString("contactPreference");
		DataObject contactDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty",
						"ContactDetails");
		customerExtn.setDataObject("contactDetails", contactDetails);

		if (planguage != null) {
			contactDetails.setString("preferredLanguage", planguage);
		}
		if (landline != null) {
			contactDetails.setString("homePhoneNumber", landline);
		}
		if (mobile != null) {
			contactDetails.setString("mobile", mobile);
		}
		if (emailId != null) {
			contactDetails.setString("email", emailId);
		}
		if (contactWhen != null) {
			contactDetails.setString("preferredTimePeriod", contactWhen);
		}
		if (contactPreference != null) {
			contactDetails.setString("preferredMedium", contactPreference);
		}
	}

	private static void customMapFromResidenceDetailsToCustomerExtension(
			DataObject residenceDetailsSrc, DataObject customerExtn) {

		String type = residenceDetailsSrc.getString("type");
		String cor = residenceDetailsSrc.getString("countryOfResidence");
		String years = residenceDetailsSrc.getString("years");
		String countryOfCitizenship = residenceDetailsSrc.getString("countryOfCitizenship");
		String status = residenceDetailsSrc.getString("status");

		DataObject residenceDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/schema/ifw/81/extn/schema/InvolvedParty",
						"ResidenceDetails");
		customerExtn.setDataObject("residenceDetails", residenceDetails);
		//System.out.println("type:"+type);
		if (type != null) {
			customerExtn.setString("residenceStatusOfUS", type);
		}

		if (cor != null) {
			residenceDetails.setString("country", cor);
		}

		if (years != null) {
			residenceDetails.setString("years", years);
		}

		if (countryOfCitizenship != null) {
			customerExtn.setString("citizenshipCountry", countryOfCitizenship);
		}
		if (status != null) {
			residenceDetails.setString("status", status);
		}

	}

	private static void customMapFromCustomerDetailsToCustomerExtension(
			DataObject customerDetailsSrc, DataObject customerExtn) {
		String title = customerDetailsSrc.getString("title");
		String fname = customerDetailsSrc.getString("fname");
		String lname = customerDetailsSrc.getString("lname");
		String faddress = customerDetailsSrc.getString("faddress");
		String saddress = customerDetailsSrc.getString("saddress");
		String city = customerDetailsSrc.getString("city");
		String state = customerDetailsSrc.getString("state");
		String suffix = customerDetailsSrc.getString("suffix");
		String zip = customerDetailsSrc.getString("zip");
		Date dateOfBirth = customerDetailsSrc.getDate("dateOfBirth");
		String gender = customerDetailsSrc.getString("gender");
		String maritalStatus = customerDetailsSrc.getString("maritalStatus");
		String ssn = customerDetailsSrc.getString("ssn");
		String id = customerDetailsSrc.getString("id");
		String status = customerDetailsSrc.getString("status");

		DataObject individualName = DataFactory.INSTANCE.create(
				"http://www.ibm.com/industries/financialservices",
				"IndividualName");

		
		if (title != null) {

			individualName.setString("prefixTitle", title);
		}
		if (fname != null) {

			individualName.setString("firstName", fname);
		}
		if (lname != null) {

			individualName.setString("lastName", lname);
		}
		if (suffix != null) {

			individualName.setString("suffixTitle", suffix);
		}
		if (faddress != null) {

			DataObject postalAddress = DataFactory.INSTANCE.create(
					"http://www.ibm.com/industries/financialservices",
					"PostalAddress");
			customerExtn.setDataObject("legalAddress", postalAddress);
			DataObject postalAddressComponent1 = DataFactory.INSTANCE.create(
					"http://www.ibm.com/industries/financialservices",
					"PostalAddressComponent");
			List addressComponentsList = new ArrayList<DataObject>();
			addressComponentsList.add(postalAddressComponent1);

			postalAddressComponent1.setString("value", faddress);
			postalAddressComponent1.setString("type", "addressLines");

			if (saddress != null) {
				DataObject postalAddressComponent2 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent2.setString("value", saddress);
				postalAddressComponent2.setString("type", "addressLines");
				addressComponentsList.add(postalAddressComponent2);
			}
			if (city != null) {
				DataObject postalAddressComponent3 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent3.setString("value", city);
				postalAddressComponent3.setString("type", "city");
				addressComponentsList.add(postalAddressComponent3);
			}

			if (state != null) {
				DataObject postalAddressComponent4 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent4.setString("value", state);
				postalAddressComponent4.setString("type", "region");
				addressComponentsList.add(postalAddressComponent4);
			}

			if (zip != null) {
				DataObject postalAddressComponent5 = DataFactory.INSTANCE
						.create(
								"http://www.ibm.com/industries/financialservices",
								"PostalAddressComponent");
				postalAddressComponent5.setString("value", zip);
				postalAddressComponent5.setString("type", "postalCode");
				addressComponentsList.add(postalAddressComponent5);
			}
			postalAddress.setList("addressComponents", addressComponentsList);
		}

		if (dateOfBirth != null) {
			
			customerExtn.setDate("dateOfBirth", dateOfBirth);
		}

		if (gender != null) {
			customerExtn.setString("gender", gender);
		}

		if (maritalStatus != null) {
			customerExtn.setString("maritalStatus", maritalStatus);
		}

		if (ssn != null) {
			customerExtn.setString("ssn", ssn);
		}
		
		if(id != null) {
			customerExtn.setString("customerId", id);
		}
		
		if(status != null) {
			customerExtn.setString("customerStatus",status);
		}
		customerExtn.setDataObject("customerName", individualName);
	}

	private static void customMapFromAccountDetailsToProductArrangementExtension(
			DataObject accountDetailsSrc, DataObject productArrangementExtn) {

		String accountType = accountDetailsSrc.getString("accountType");
		String product = accountDetailsSrc.getString("product");
		String accountNo = accountDetailsSrc.getString("accountno");
		Date doo = accountDetailsSrc.getDate("dateOfOpening");

		if (accountType != null) {
			productArrangementExtn.setString("accountType", accountType);
		}
		if (product != null) {
			productArrangementExtn.setString("product", product);
		}

		if(accountNo != null) {
			productArrangementExtn.setString("accountNumber", accountNo);
		}
		
		if(doo != null) {
			productArrangementExtn.setDate("startDate", doo);
		}
		String type = productArrangementExtn.getType().getName();

		if (type.equals("DirectDepositArrangementExtn")) {
			customMapFromAccountToDirectDepositArrangementExtension(
					accountDetailsSrc, productArrangementExtn);
		}
	}

	private static void customMapFromAccountToDirectDepositArrangementExtension(
			DataObject accountDetailsSrc,
			DataObject directDepositArrangementExtn) {
		DataObject cdDetailsSrc = accountDetailsSrc.getDataObject("cdDetails");
		if (cdDetailsSrc != null) {
			DataObject term = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/banking/schema/ifw/81/extn/schema/Arrangement",
							"Term");
			directDepositArrangementExtn.setDataObject("term", term);
			DataObject termSrc = cdDetailsSrc.getDataObject("term");
			if (termSrc != null) {
				String year = termSrc.getString("year");
				String month = termSrc.getString("month");
				String day = termSrc.getString("day");
				if (year != null) {
					term.setString("years", year);
				}
				if (month != null) {
					term.setString("months", month);
				}
				if (day != null) {
					term.setString("days", day);
				}
			}
			DataObject currencyAmount = DataFactory.INSTANCE.create(
					"http://www.ibm.com/industries/financialservices",
					"CurrencyAmount");
			Double theCurrencyAmount = cdDetailsSrc.getDouble("amount");
			String theCurrencyCode = cdDetailsSrc.getString("currency");

			if (theCurrencyAmount != null) {
				currencyAmount
						.setDouble("theCurrencyAmount", theCurrencyAmount);
			}
			if (theCurrencyCode != null) {
				currencyAmount.setString("theCurrencyCode", theCurrencyCode);
			}
			directDepositArrangementExtn.setDataObject("depositAmount",
					currencyAmount);
			Date date = cdDetailsSrc.getDate("date");
			if (date != null) {
				directDepositArrangementExtn.setDate("depositDate", date);
			}

		}
	}

}
