/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
/***********************************************
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.icp.banking.custom.map;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import com.ibm.websphere.sca.sdo.DataFactory;
import commonj.sdo.DataObject;

public class CustomProductArrangementToAccountDataObjectMapper {

	private CustomProductArrangementToAccountDataObjectMapper() {

	}

	public static void customMapFromProductArrangementExtensionToAccount(
			DataObject productArrangementExtnSrc, DataObject account)
			throws MapException {

		if (productArrangementExtnSrc == null || account == null) {
			throw new MapException("Source or target can not be Null Target ");
		}

		DataObject customerExtn = productArrangementExtnSrc
				.getDataObject("customer");

		if (customerExtn != null) {

			DataObject customer = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"Customer");
			account.setDataObject("customer", customer);
			mapCustomerExtnToCustomer(customerExtn, customer);
			mapAccountDetailsFromProductArrangementExtnToAccount(
					productArrangementExtnSrc, account);
			mapDocumentsFromProductExtnToDocument(productArrangementExtnSrc,account);
		}
		
	

	}

	public static void mapDocumentsFromProductExtnToDocument(
			DataObject productArrangementExtnSrc, DataObject account) {
		List docList = productArrangementExtnSrc.getList("documents");
		if(docList != null) {
			DataObject document = DataFactory.INSTANCE.create("http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema", "Documents");
			if(!docList.isEmpty()) {
				for (Iterator iterator = docList.iterator(); iterator.hasNext();) {
					String doc = (String) iterator.next();
					if(doc != null) {
						if(doc.equalsIgnoreCase("Pay stubs")) {
							document.setBoolean("payStubsEnabled", true);
						} else 
						if(doc.equalsIgnoreCase("W2 form")) {
							document.setBoolean("w2formEnabled", true);
						} else
						if(doc.equalsIgnoreCase("Two years of income tax returns")) {
							document.setBoolean("twoYearsIncomeTaxReturnsEnabled", true);
						} else 
						if(doc.equalsIgnoreCase("Tax return")) {
							document.setBoolean("taxReturnEnabled", true);
						} else 
						if(doc.equalsIgnoreCase("SSN Card")) {
							document.setBoolean("ssnCardEnabled", true);
						} else 	
						if(doc.equalsIgnoreCase("Driving License")) {
							document.setBoolean("drivingLicenseEnabled", true);
						} else 
						if(doc.equalsIgnoreCase("Proof of Citizenship/PR")) {
							document.setBoolean("proofOfCitizenshipEnabled", true);
						} else 
						if(doc.equalsIgnoreCase("Visa Information")) {
							document.setBoolean("visaInfoEnabled", true);
						}
					}
				}
				account.setDataObject("document", document);
			}
		}
		
	}

	private static void customMapFromDirectDepositArrangementExtensionToAccount(
			DataObject productArrangementExtnSrc, DataObject accountDetails) {

		DataObject termSrc = productArrangementExtnSrc.getDataObject("term");

		if (termSrc != null && accountDetails != null) {
			int years = termSrc.getInt("years");
			int months = termSrc.getInt("months");
			int days = termSrc.getInt("days");

			DataObject CDDetails = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"CDDetails");
			accountDetails.setDataObject("cdDetails", CDDetails);

			DataObject term = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"Term");

			CDDetails.setDataObject("term", term);
			if (new Integer(years) != null) {
				term.setInt("year", years);

			}

			if (new Integer(months) != null) {
				term.setInt("month", months);

			}
			if (new Integer(days) != null) {
				term.setInt("day", days);
			}

			DataObject depositAmount = productArrangementExtnSrc
					.getDataObject("depositAmount");
			if (depositAmount != null) {
				Double amount = depositAmount.getDouble("theCurrencyAmount");
				String code = depositAmount.getString("theCurrencyCode");

				if (amount != null) {
					CDDetails.setDouble("amount", amount);
				}
				if (code != null) {
					CDDetails.setString("currency", code);
				}
			}

			Date depositDate = productArrangementExtnSrc.getDate("depositDate");
			if (depositDate != null) {
				CDDetails.setDate("date", depositDate);
			}
		}

	}

	private static void mapAccountDetailsFromProductArrangementExtnToAccount(
			DataObject productArrangementExtnSrc, DataObject account) {

		String accountType = productArrangementExtnSrc.getString("accountType");
		String product = productArrangementExtnSrc.getString("product");
		String accountNo = productArrangementExtnSrc.getString("accountNumber");
		String applicationNo = productArrangementExtnSrc.getString("applicationNumber");
		Date doo = productArrangementExtnSrc.getDate("startDate");
		String operatingInstruction = productArrangementExtnSrc
				.getString("operatingInstruction");
		String card = productArrangementExtnSrc.getString("card");
		Boolean internetBanking = productArrangementExtnSrc
				.getBoolean("internetBanking");
		Boolean ivr = productArrangementExtnSrc.getBoolean("ivr");
		Boolean checkBooks = productArrangementExtnSrc.getBoolean("checkBook");
		int quantity = productArrangementExtnSrc.getInt("checkQuantity");

		DataObject accountDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
						"AccountDetails");

		account.setDataObject("accountDetails", accountDetails);

		if(doo != null) {
			accountDetails.setDate("dateOfOpening",doo);
		}
		if(applicationNo != null) {
			account.setString("applicationno",applicationNo);
		}
		if (accountType != null) {
			accountDetails.setString("accountType", accountType);
		}
		if (product != null) {
			accountDetails.setString("product", product);
		}
		if(accountNo != null) {
			accountDetails.setString("accountno", accountNo);
		}

		if (operatingInstruction != null) {
			account.setString("operatingInstructions", operatingInstruction);
		}

		if (card != null) {
			account.setString("card", card);
		}

		if (internetBanking != null) {
			if(internetBanking) {
				account.setString("internetBanking", "Yes");
			} else {
				{
					account.setString("internetBanking", "No");
				}
			}
			
		}

		if (ivr != null) {
			if(ivr) {
				account.setString("IVR", "Yes");
			} else {
				account.setString("IVR", "No");
			}
			
		}

		if (checkBooks != null) {
			if(checkBooks) {
				account.setString("checks", "Yes");
				if(quantity > 0) {
					account.setInt("quantity", quantity);
				}
				
			} else {
				account.setString("checks", "No");
			}
			
		}

		if (new Integer(quantity) != null || quantity > 0) {
			account.setInt("quantity", quantity);
		}

		DataObject accountStatement = productArrangementExtnSrc
				.getDataObject("statement");

		if (accountStatement != null) {
			String type = accountStatement.getString("type");
			String frequency = accountStatement.getString("frequency");
			if (type != null) {
				account.setString("accountStatement", type);
			}
			if (frequency != null) {
				account.setString("subscriptionType", frequency);
			}

		}
		
		List coApplicantsList = productArrangementExtnSrc.getList("coApplicants");
		if(coApplicantsList != null) {
			DataObject coApplicantInformation = DataFactory.INSTANCE.create(
					"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema","CoApplicantInformation");
			account.setDataObject("coApplicantInformation",coApplicantInformation);
			List coApplicantInformationList = new ArrayList();
			if(!coApplicantsList.isEmpty()) {
			coApplicantInformation.setString("coApplicant","Yes");
			
			for (Iterator iterator = coApplicantsList.iterator(); iterator
					.hasNext();) {
				DataObject coApplicantDO = DataFactory.INSTANCE.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema","CoApplicant");
				DataObject coApplicants = (DataObject) iterator.next();
				if(coApplicants != null) {
					int age = coApplicants.getInt("age");
					//System.out.println("age"+age);
					if(age > 0) {
						coApplicantDO.setFloat("age",age);
					}
				}
				DataObject legalAddress = coApplicants.getDataObject("legalAddress");
				if(legalAddress != null) {
					List addressComponentsList = legalAddress.getList("addressComponents");
					for (Iterator iterator2 = addressComponentsList.iterator(); iterator2
							.hasNext();) {
						DataObject addressComponent = (DataObject) iterator2.next();
						if(addressComponent != null) {
							String type = addressComponent.getString("type");
							String value = addressComponent.getString("value");
							//System.out.println("value: "+value);
							
							if(type != null && value != null) {
								if(type.equals("addressLines")) {
									coApplicantDO.setString("address",value);
									break;
								}
							}
						}
						
					}
				}
				DataObject customerName = coApplicants.getDataObject("customerName");
				if(customerName != null) {
					
					String name = customerName.getString("firstName");
					if(name != null) {
						
							coApplicantDO.setString("name", name);
						}
					
				}
				//System.out.println("coApplicantDO:"+coApplicantDO);
				coApplicantInformationList.add(coApplicantDO);
			}
			coApplicantInformation.setList("coApplicantList", coApplicantInformationList);
			
			//System.out.println("coApplicantInformationList:"+coApplicantInformationList);
		}
			else {
				coApplicantInformation.setString("coApplicant","No");
			}
		}

		String type = productArrangementExtnSrc.getType().getName();

		if (type.equals("DirectDepositArrangementExtn")) {
			customMapFromDirectDepositArrangementExtensionToAccount(
					productArrangementExtnSrc, accountDetails);
		}

	}

	public static void mapCustomerExtnToCustomer(DataObject customerExtn,
			DataObject customer) {

		DataObject individualName = customerExtn.getDataObject("customerName");

		DataObject customerDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
						"CustomerDetails");
		customer.setDataObject("customerDetails", customerDetails);

		if (individualName != null) {
			mapIndividualNameToCustomerDetails(individualName, customerDetails);
		}

		DataObject legalAddress = customerExtn.getDataObject("legalAddress");

		if (legalAddress != null) {
			mapLegalAddressToCustomerDetails(legalAddress, customerDetails);
		}

		DataObject contactDetailsSrc = customerExtn
				.getDataObject("contactDetails");

		if (contactDetailsSrc != null) {
			DataObject contactDetails = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"ContactDetails");
			customer.setDataObject("contactDetails", contactDetails);
			mapContactDetailsFromCustomerExtnToContactDetails(
					contactDetailsSrc, contactDetails);
		}

		DataObject residenceDetailsSrc = customerExtn
				.getDataObject("residenceDetails");
		if (residenceDetailsSrc != null) {
			DataObject residenceDetails = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"ResidenceDetails");
			mapResidenceDetailsFromCustomerExtnToResidenceDetails(
					residenceDetailsSrc, residenceDetails);
			String countryOfCitizenship = customerExtn.getString("citizenshipCountry");
			String type = customerExtn.getString("residenceStatusOfUS");
			if (countryOfCitizenship != null) {
				residenceDetails.setString("countryOfCitizenship", countryOfCitizenship);
			}
			if (type != null) {
				residenceDetails.setString("type", type);
			}
			customer.setDataObject("residenceDetails", residenceDetails);
		}

		DataObject employmentDetailsSrc = customerExtn
				.getDataObject("employmentDetails");
		if (employmentDetailsSrc != null) {
			DataObject employmentDetails = DataFactory.INSTANCE
					.create(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
							"EmploymentDetails");
			mapEmploymentDetailsFromCustomerExtnToEmploymentDetails(
					employmentDetailsSrc, employmentDetails);
			customer.setDataObject("employmentDetails", employmentDetails);

		}

		Date dateOfBirth = customerExtn.getDate("dateOfBirth");
		if (dateOfBirth != null) {
			customerDetails.setDate("dateOfBirth", dateOfBirth);
		}

		String gender = customerExtn.getString("gender");
		if (gender != null) {
			customerDetails.setString("gender", gender);
		}

		String maritalStatus = customerExtn.getString("maritalStatus");
		if (maritalStatus != null) {
			customerDetails.setString("maritalStatus", maritalStatus);
		}

		String ssn = customerExtn.getString("ssn");
		if (ssn != null) {
			customerDetails.setString("ssn", ssn);
		}

		String id = customerExtn.getString("customerId");
		if (id != null) {
			customerDetails.setString("id", id);
		}

		String customerStatus = customerExtn.getString("customerStatus");
		if (customerStatus != null) {
			customerDetails.setString("status", customerStatus);
		}

		String rManager = customerExtn.getString("relationshipManager");
		String rCode = customerExtn.getString("relationshipCode");

		DataObject relationshipDetails = DataFactory.INSTANCE
				.create(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2010/05/banking/payments/schema",
						"RelationshipDetails");

		customer.setDataObject("relationshipDetails", relationshipDetails);

		if (rManager != null) {
			relationshipDetails.setString("relationshipManager", rManager);
		}
		if (rCode != null) {
			relationshipDetails.setString("relationshipCode", rCode);
		}

	}

	private static void mapEmploymentDetailsFromCustomerExtnToEmploymentDetails(
			DataObject employmentDetailsSrc, DataObject employmentDetails) {
		String type = employmentDetailsSrc.getString("type");
		long annualIncome = employmentDetailsSrc.getLong("annualIncome");
		String employerName = employmentDetailsSrc.getString("employerName");
		String position = employmentDetailsSrc.getString("position");
		String homePhoneNumber = employmentDetailsSrc
				.getString("homePhoneNumber");
		String mobile = employmentDetailsSrc.getString("mobile");

		if (type != null) {
			employmentDetails.setString("employement", type);
		}

		if (new Long(annualIncome) != null) {
			employmentDetails.setLong("annualIncome", annualIncome);
		}

		if (employerName != null) {
			employmentDetails.setString("name", employerName);
		}

		if (position != null) {
			employmentDetails.setString("position", position);
		}

		if (homePhoneNumber != null) {
			employmentDetails.setString("landline", homePhoneNumber);
		}
		if (mobile != null) {
			employmentDetails.setString("mobile", mobile);
		}

		DataObject legalAddress = employmentDetailsSrc
				.getDataObject("legalAddress");

		if (legalAddress != null) {
			List addressComponentsList = legalAddress
					.getList("addressComponents");
			int i = 0;
			if (addressComponentsList != null) {
				if (addressComponentsList.size() > 0) {
					for (Iterator iterator = addressComponentsList.iterator(); iterator
							.hasNext();) {
						DataObject addressComponents = (DataObject) iterator
								.next();
						String value = addressComponents.getString("value");
						String addressType = addressComponents
								.getString("type");

						if (value != null && addressType != null) {
							if (addressType.equals("addressLines")) {
								if (i == 0) {
									employmentDetails.setString("faddress",
											value);
								}
								if (i == 1) {
									employmentDetails.setString("saddress",
											value);
								}
							} else if (addressType.equals("city")) {
								employmentDetails.setString("city", value);
							} else if (addressType.equals("region")) {
								employmentDetails.setString("state", value);
							} else if (addressType.equals("postalCode")) {
								employmentDetails.setString("zip", value);
							}
						}
						i++;
					}
				}
			}
		}

	}

	private static void mapResidenceDetailsFromCustomerExtnToResidenceDetails(
			DataObject residenceDetailsSrc, DataObject residenceDetails) {
		String country = residenceDetailsSrc.getString("country");
		String years = residenceDetailsSrc.getString("years");
		String status = residenceDetailsSrc.getString("status");

		if (country != null) {
			residenceDetails.setString("countryOfResidence", country);
		}
		if (years != null) {
			residenceDetails.setString("years", years);
		}
		if (status != null) {
			residenceDetails.setString("status", status);
		}
	}

	private static void mapContactDetailsFromCustomerExtnToContactDetails(
			DataObject contactDetailsSrc, DataObject contactDetails) {

		String homePhoneNumber = contactDetailsSrc.getString("homePhoneNumber");
		String mobile = contactDetailsSrc.getString("mobile");
		String email = contactDetailsSrc.getString("email");
		String preferredLanguage = contactDetailsSrc
				.getString("preferredLanguage");
		String preferredMedium = contactDetailsSrc.getString("preferredMedium");
		String preferredTimePeriod = contactDetailsSrc
				.getString("preferredTimePeriod");

		if (homePhoneNumber != null) {
			contactDetails.setString("landline", homePhoneNumber);
		}
		if (mobile != null) {
			contactDetails.setString("mobile", mobile);
		}
		if (email != null) {
			contactDetails.setString("emailId", email);
		}
		if (preferredLanguage != null) {
			contactDetails.setString("preferredLanguage", preferredLanguage);
		}
		if (preferredMedium != null) {
			contactDetails.setString("contactPreference", preferredMedium);
		}
		if (preferredTimePeriod != null) {
			contactDetails.setString("contactWhen", preferredTimePeriod);
		}
	}

	private static void mapLegalAddressToCustomerDetails(
			DataObject legalAddress, DataObject customerDetails) {
		List addressComponentsList = legalAddress.getList("addressComponents");
		if (addressComponentsList != null) {
			int i = 0;
			for (Iterator iterator = addressComponentsList.iterator(); iterator
					.hasNext();) {
				DataObject addressComponent = (DataObject) iterator.next();
				String value = addressComponent.getString("value");
				String type = addressComponent.getString("type");

				if (type != null && value != null) {
					if (type.equals("addressLines")) {
						if (i == 0) {
							customerDetails.setString("faddress", value);
						} else if (i == 1) {
							customerDetails.setString("saddress", value);
						}
					} else if (type.equals("city")) {
						customerDetails.setString("city", value);
					} else if (type.equals("region")) {
						customerDetails.setString("state", value);
					} else if (type.equals("postalCode")) {
						customerDetails.setString("zip", value);
					}
				}
				i++;
			}
		}

	}

	private static void mapIndividualNameToCustomerDetails(
			DataObject individualName, DataObject customerDetails) {

		String firstName = individualName.getString("firstName");
		String lastName = individualName.getString("lastName");
		String prefixTitle = individualName.getString("prefixTitle");
		String suffixTitle = individualName.getString("suffixTitle");

		if (firstName != null) {
			customerDetails.setString("fname", firstName);
		}

		if (lastName != null) {
			customerDetails.setString("lname", lastName);
		}

		if (prefixTitle != null) {
			customerDetails.setString("title", prefixTitle);
		}

		if (suffixTitle != null) {
			customerDetails.setString("suffix", suffixTitle);
		}

	}

}
